#!/usr/bin/env python
'''=================================================================================================
Infer RNA-seq experiment design from SAM/BAM file. This module will determine if the RNA-seq
experiment is:
1) pair-end or single-end
2) if experiment is strand-specific, how reads were stranded.
 * For pair-end RNA-seq, there are two different ways to strand reads:
  i) 1++,1--,2+-,2-+
     read1 mapped to '+' strand indicates parental gene on '+' strand
     read1 mapped to '-' strand indicates parental gene on '-' strand
     read2 mapped to '+' strand indicates parental gene on '-' strand
     read2 mapped to '-' strand indicates parental gene on '+' strand
  ii) 1+-,1-+,2++,2--
     read1 mapped to '+' strand indicates parental gene on '-' strand
     read1 mapped to '-' strand indicates parental gene on '+' strand
     read2 mapped to '+' strand indicates parental gene on '+' strand
     read2 mapped to '-' strand indicates parental gene on '-' strand
 * For single-end RNA-seq, there are two different ways to strand reads:
  i) ++,--
     read mapped to '+' strand indicates parental gene on '+' strand
     read mapped to '-' strand indicates parental gene on '-' strand
  ii) +-,-+
     read mapped to '+' strand indicates parental gene on '-' strand
     read mapped to '-' strand indicates parental gene on '+' strand		
 
 NOTE:
	You don't need to know the RNA sequencing protocol before mapping your reads to the reference
 	genome. Mapping your RNA-seq reads as if they were non-strand specific, this script can
 	"guess" how RNA-seq reads were stranded.
================================================================================================='''

#import built-in modules
import os,sys
if sys.version_info[0] != 3:
	print("\nYou are using python" + str(sys.version_info[0]) + '.' + str(sys.version_info[1]) + " This verion of RSeQC needs python3!\n", file=sys.stderr)
	sys.exit()	

import re
import string
from optparse import OptionParser
import warnings
import string
import collections
import math
from time import strftime

#import third-party modules
from bx.bitset import *
from bx.bitset_builders import *
from bx.intervals import *

#import my own modules
from qcmodule import SAM
#changes to the paths

#changing history to this module


__author__ = "Liguo Wang"
__copyright__ = "Copyleft"
__credits__ = []
__license__ = "GPL"
__version__="3.0.0"
__maintainer__ = "Liguo Wang"
__email__ = "wang.liguo@mayo.edu"
__status__ = "Production"


def printlog (mesg):
	'''print progress into stderr and log file'''
	mesg="@ " + strftime("%Y-%m-%d %H:%M:%S") + ": " + mesg
	LOG=open('class.log','a')
	print(mesg, file=sys.stderr)
	print(mesg, file=LOG)


def main():
	usage="%prog [options]" + "\n"
	parser = OptionParser(usage,version="%prog " + __version__)
	parser.add_option("-i","--input-file",action="store",type="string",dest="input_file",help="Input alignment file in SAM or BAM format")
	parser.add_option("-r","--refgene",action="store",type="string",dest="refgene_bed",help="Reference gene model in bed fomat.")
	parser.add_option("-s","--sample-size",action="store",type="int",dest="sample_size",default=200000, help="Number of reads sampled from SAM/BAM file. default=%default")	
	parser.add_option("-q","--mapq",action="store",type="int",dest="map_qual",default=30,help="Minimum mapping quality (phred scaled) for an alignment to be considered as \"uniquely mapped\". default=%default")

	(options,args)=parser.parse_args()

	if not (options.input_file and options.refgene_bed):
		parser.print_help()
		print('\n\n' + __doc__, file=sys.stderr)
		sys.exit(0)
	for f in (options.input_file,options.refgene_bed):
		if not os.path.exists(f):
			print('\n\n' + f + " does NOT exists." + '\n', file=sys.stderr)
			sys.exit(0)
	if options.sample_size <1000:
		print("Warn: Sample Size too small to give a accurate estimation", file=sys.stderr)
	obj = SAM.ParseBAM(options.input_file)
	(protocol,sp1,sp2,other)=obj.configure_experiment(refbed=options.refgene_bed, sample_size = options.sample_size, q_cut = options.map_qual)
	if other <0: other=0.0
	if protocol == "PairEnd":
		print("\n\nThis is PairEnd Data")
		print("Fraction of reads failed to determine: %.4f" % other)
		print("Fraction of reads explained by \"1++,1--,2+-,2-+\": %.4f" % sp1)
		print("Fraction of reads explained by \"1+-,1-+,2++,2--\": %.4f" % sp2)
		
	elif protocol == "SingleEnd":
		print("\n\nThis is SingleEnd Data")
		print("Fraction of reads failed to determine: %.4f" % other)
		print("Fraction of reads explained by \"++,--\": %.4f" % sp1)
		print("Fraction of reads explained by \"+-,-+\": %.4f" % sp2)
		
	else:
		print("Unknown Data type")
	#print mesg

if __name__ == '__main__':
	main()



#!/usr/bin/env python
'''
Calculate the RNA-seq reads coverage over gene body. 

Note:
1) Only input sorted and indexed BAM file(s). SAM format is not supported.
2) Genes/transcripts with mRNA length < 100 will be skipped (Number specified to "-l" cannot be < 100). 
'''

#import built-in modules
import os,sys
if sys.version_info[0] != 3:
	print("\nYou are using python" + str(sys.version_info[0]) + '.' + str(sys.version_info[1]) + " This verion of RSeQC needs python3!\n", file=sys.stderr)
	sys.exit()	

import re
import string
from optparse import OptionParser
import warnings
import collections
import math
from time import strftime
import subprocess
from os.path import basename
import operator

#import third-party modules
from numpy import std,mean
from bx.bitset import *
from bx.bitset_builders import *
from bx.intervals import *
import pysam

#import my own modules
from qcmodule import getBamFiles
from qcmodule import mystat
#changes to the paths

#changing history to this module


__author__ = "Liguo Wang"
__copyright__ = "Copyleft"
__credits__ = []
__license__ = "GPL"
__version__="3.0.0"
__maintainer__ = "Liguo Wang"
__email__ = "wang.liguo@mayo.edu"
__status__ = "Production"


def valid_name(s):
	'''make sure the string 's' is valid name for R variable'''
	symbols = 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789_.'
	digit = '0123456789'
	rid = '_'.join(i for i in s.split())	#replace space(s) with '_'
	if rid[0] in digit:rid = 'V' + rid
	tmp = ''
	for i in rid:
		if i in symbols:
			tmp = tmp + i
		else:
			tmp = tmp + '_'
	return tmp
	

def printlog (mesg):
	'''print progress into stderr and log file'''
	mesg="@ " + strftime("%Y-%m-%d %H:%M:%S") + ": " + mesg
	LOG=open('log.txt','a')
	print(mesg, file=sys.stderr)
	print(mesg, file=LOG)


def pearson_moment_coefficient(lst):
	'''measure skewness'''
	mid_value = lst[int(len(lst)/2)]
	sigma = std(lst, ddof=1)
	tmp = []
	for i in lst:
		tmp.append(((i - mid_value)/sigma)**3)
	return mean(tmp)

def genebody_percentile(refbed, mRNA_len_cut = 100):
	'''
	return percentile points of gene body
	mRNA length < mRNA_len_cut will be skipped
	'''
	if refbed is None:
		print("You must specify a bed file representing gene model\n", file=sys.stderr)
		exit(0)
	
	g_percentiles = {}
	transcript_count = 0
	for line in open(refbed,'r'):
		try:
			if line.startswith(('#','track','browser')):continue  
			# Parse fields from gene tabls
			fields = line.split()
			chrom     = fields[0]
			tx_start  = int( fields[1] )
			tx_end    = int( fields[2] )
			geneName      = fields[3]
			strand    = fields[5]
			geneID = '_'.join([str(j) for j in (chrom, tx_start, tx_end, geneName, strand)])
				
			exon_starts = list(map( int, fields[11].rstrip( ',\n' ).split( ',' ) ))
			exon_starts = list(map((lambda x: x + tx_start ), exon_starts))
			exon_ends = list(map( int, fields[10].rstrip( ',\n' ).split( ',' ) ))
			exon_ends = list(map((lambda x, y: x + y ), exon_starts, exon_ends))
			transcript_count += 1
		except:
			print("[NOTE:input bed must be 12-column] skipped this line: " + line, end=' ', file=sys.stderr)
			continue
		gene_all_base=[]
		mRNA_len =0
		flag=0
		for st,end in zip(exon_starts,exon_ends):
			gene_all_base.extend(list(range(st+1,end+1)))		#1-based coordinates on genome
		if len(gene_all_base) < mRNA_len_cut:
			continue
		g_percentiles[geneID] = (chrom, strand, mystat.percentile_list (gene_all_base))	#get 100 points from each gene's coordinates
	printlog("Total " + str(transcript_count) + ' transcripts loaded')
	return g_percentiles

def genebody_coverage(bam, position_list):
	'''
	position_list is dict returned from genebody_percentile
	position is 1-based genome coordinate
	'''
	samfile = pysam.Samfile(bam, "rb")
	aggreagated_cvg = collections.defaultdict(int)
	
	gene_finished = 0
	for chrom, strand, positions in list(position_list.values()):
		coverage = {}
		for i in positions:
			coverage[i] = 0.0
		chrom_start = positions[0]-1
		if chrom_start <0: chrom_start=0
		chrom_end = positions[-1]
		try:
			samfile.pileup(chrom, 1,2)
		except:
			continue
			
		for pileupcolumn in samfile.pileup(chrom, chrom_start, chrom_end, truncate=True):
			ref_pos = pileupcolumn.pos+1
			if ref_pos not in positions:
				continue
			if pileupcolumn.n == 0:
				coverage[ref_pos] = 0
				continue				
			cover_read = 0
			for pileupread in pileupcolumn.pileups:
				if pileupread.is_del: continue
				if pileupread.alignment.is_qcfail:continue 
				if pileupread.alignment.is_secondary:continue 
				if pileupread.alignment.is_unmapped:continue
				if pileupread.alignment.is_duplicate:continue
				cover_read +=1
			coverage[ref_pos] = cover_read
		tmp = [coverage[k] for k in sorted(coverage)]
		if strand == '-':
			tmp = tmp[::-1]
		for i in range(0,len(tmp)):
			aggreagated_cvg[i] += tmp[i]
		gene_finished += 1
		
		if gene_finished % 100 == 0:
			print("\t%d transcripts finished\r" % (gene_finished), end=' ', file=sys.stderr)
	return 	aggreagated_cvg
		
def Rcode_write(dataset,file_prefix, format='pdf', colNum=100):
	'''generate R script for visualization'''
	ROUT = open(file_prefix + '.r','w')
	names=[]
	datas=[]
	for name, data, tmp in dataset:
		names.append(name)
		datas.append(data)
		print(name + ' <- c(' + ','.join([str(i) for i in data]) + ')', file=ROUT)
		
	tick_pos = [1,10,20,30,40,50,60,70,80,90,100]
	tick_lab = [1,10,20,30,40,50,60,70,80,90,100]
	
	# do not generate heatmap if only 1 sample
	if len(names) >=3:
		print('data_matrix' + ' <- matrix(c(' + ','.join(names) + '), byrow=T, ' +  'ncol=' + str(colNum) + ')', file=ROUT)
		print('rowLabel <- c(' + ','.join(['"' + i + '"' for i in names]) + ')', file=ROUT)
		print('\n', file=ROUT)
		print('%s(\"%s.%s\")' % (format.lower(),file_prefix + ".heatMap",format.lower()), file=ROUT)
		print('rc <- cm.colors(ncol(data_matrix))', file=ROUT)
		print('heatmap(data_matrix' + ', scale=c(\"none\"),keep.dendro=F, labRow = rowLabel ' + ',Colv = NA,Rowv = NA,labCol=NA,col=cm.colors(256),margins = c(6, 8),ColSideColors = rc,cexRow=1,cexCol=1,xlab="Gene body percentile (5\'->3\')", add.expr=x_axis_expr <- axis(side=1,at=c(%s),labels=c(%s)))' % (','.join([str(i) for i in tick_pos]), ','.join(['"' + str(i) + '"' for i in tick_lab])), file=ROUT)
		print('dev.off()', file=ROUT)
    
    
	print('\n', file=ROUT)
	
	print('%s(\"%s.%s\")' % (format.lower(),file_prefix + ".curves",format.lower()), file=ROUT)	
	print("x=1:%d" % (colNum), file=ROUT) 
	print('icolor = colorRampPalette(c("#7fc97f","#beaed4","#fdc086","#ffff99","#386cb0","#f0027f"))(%d)' % (len(names)), file=ROUT)
	
	if len(names) == 1:
		print("plot(x,%s,type='l',xlab=\"Gene body percentile (5\'->3\')\", ylab=\"Coverage\",lwd=0.8,col=icolor[1])" % (names[0]), file=ROUT)
	
	elif  len(names) >=2 and len(names) <=6:  
		print("plot(x,%s,type='l',xlab=\"Gene body percentile (5\'->3\')\", ylab=\"Coverage\",lwd=0.8,col=icolor[1])" % (names[0]), file=ROUT)
		for i in range(1,len(names)):
			print("lines(x,%s,type='l',col=icolor[%d])" % (names[i], i+1), file=ROUT)
		print("legend(0,1,fill=icolor[%d:%d], legend=c(%s))" % (1,len(names), ','.join([ "'" + str(n) + "'" for n in names])), file=ROUT)
	
	elif len(names) > 6:
		print('layout(matrix(c(1,1,1,2,1,1,1,2,1,1,1,2), 4, 4, byrow = TRUE))', file=ROUT)
		print("plot(x,%s,type='l',xlab=\"Gene body percentile (5\'->3\')\", ylab=\"Coverage\",lwd=0.8,col=icolor[1])" % (names[0]), file=ROUT)
		for i in range(1,len(names)):
			print("lines(x,%s,type='l',col=icolor[%d])" % (names[i], i+1), file=ROUT)
		print('par(mar=c(1,0,2,1))', file=ROUT)
		print('plot.new()', file=ROUT)
		print("legend(0,1,fill=icolor[%d:%d],legend=c(%s))" % (1,len(names), ','.join([ "'" + str(n) + "'" for n in names])), file=ROUT)

	print('dev.off()', file=ROUT)
	ROUT.close()

def main():
	usage="%prog [options]" + '\n' + __doc__ + "\n"
	parser = OptionParser(usage,version="%prog " + __version__)
	parser.add_option("-i","--input",action="store",type="string",dest="input_files",help='Input file(s) in BAM format. "-i" takes these input: 1) a single BAM file. 2) "," separated BAM files. 3) directory containing one or more bam files. 4) plain text file containing the path of one or more bam file (Each row is a BAM file path). All BAM files should be sorted and indexed using samtools.')
	parser.add_option("-r","--refgene",action="store",type="string",dest="ref_gene_model",help="Reference gene model in bed format. [required]")
	parser.add_option("-l","--minimum_length",action="store",type="int",default=100, dest="min_mRNA_length",help="Minimum mRNA length (bp). mRNA smaller than \"min_mRNA_length\" will be skipped. default=%default")
	parser.add_option("-f","--format",action="store",type="string",dest="output_format", default='pdf', help="Output file format, 'pdf', 'png' or 'jpeg'. default=%default")
	parser.add_option("-o","--out-prefix",action="store",type="string",dest="output_prefix",help="Prefix of output files(s). [required]")
	(options,args)=parser.parse_args()

	if not (options.output_prefix and options.input_files and options.ref_gene_model):
		parser.print_help()
		sys.exit(0)

	if not os.path.exists(options.ref_gene_model):
		print('\n\n' + options.ref_gene_model + " does NOT exists" + '\n', file=sys.stderr)
		#parser.print_help()
		sys.exit(0)
	if options.min_mRNA_length < 100:
		print('The number specified to "-l" cannot be smaller than 100.' + '\n', file=sys.stderr)
		sys.exit(0)
		
	OUT1 = open(options.output_prefix  + ".geneBodyCoverage.txt"	,'w')
	print("Percentile\t" + '\t'.join([str(i) for i in range(1,101)]), file=OUT1)
		
	printlog("Read BED file (reference gene model) ...")
	gene_percentiles = genebody_percentile(refbed = options.ref_gene_model, mRNA_len_cut = options.min_mRNA_length)
		
	printlog("Get BAM file(s) ...")
	bamfiles = getBamFiles.get_bam_files(options.input_files)
	for f in bamfiles:
		print("\t" + f, file=sys.stderr)
	
	file_container = []
	for bamfile in bamfiles:
		printlog("Processing " + basename(bamfile) + ' ...')
		cvg = genebody_coverage(bamfile, gene_percentiles)
		if len(cvg) == 0:
			print("\nCannot get coverage signal from " + basename(bamfile) + ' ! Skip', file=sys.stderr)
			continue
		tmp = valid_name(basename(bamfile).replace('.bam',''))	# scrutinize R identifer
		if file_container.count(tmp) == 0:
			print(tmp + '\t' + '\t'.join([str(cvg[k]) for k in sorted(cvg)]), file=OUT1)
		else:
			print(tmp + '.' + str(file_container.count(tmp)) + '\t' + '\t'.join([str(cvg[k]) for k in sorted(cvg)]), file=OUT1)
		file_container.append(tmp)
	OUT1.close()
	
	
	dataset=[]
	for line in open(options.output_prefix  + ".geneBodyCoverage.txt",'r'):
		line = line.strip()
		if line.startswith("Percentile"):
			continue
		f = line.split()
		name = f[0]
		dat = [float(i) for i in  f[1:]]
		skewness = pearson_moment_coefficient(dat)
		dataset.append((name, [(i -min(dat))/(max(dat) - min(dat)) for i in dat], skewness))	
	dataset.sort(key = operator.itemgetter(2), reverse=True)
	
	print("\n\n", file=sys.stderr)
	print("\tSample\tSkewness", file=sys.stderr)
	for a,b,c in dataset:
		print('\t' + a + '\t' + str(c), file=sys.stderr)
	Rcode_write(dataset, options.output_prefix + '.geneBodyCoverage', format = options.output_format)
	
	printlog("Running R script ...")
	try:
		subprocess.call("Rscript " + options.output_prefix + '.geneBodyCoverage.r',shell=True)
	except:
		print("Cannot generate pdf file from " + options.output_prefix + '.geneBodyCoverage.r', file=sys.stderr)
		pass
	
	
if __name__ == '__main__':
	main()
	


#!/usr/bin/env python

'''manipulate CIGAR string represented as list of tuples (BAM file)
BAM OP  Description
0   M   alignment match
1   I   insertion to read. (relative to reference genome)
2   D   deletion from read. (relative to reference genome)
3   N   skipped region from the reference
4   S   soft clipping (clipped sequence present in SEQ)
5   H   hard clipping (clipped sequences NOT present in SEQ) 
6   P   padding (silent deletion from padded reference)
7   =   sequence match
8   X   sequence mismatch

Example:
The tuple [ (0,3), (1,5), (0,2) ] refers to an alignment with 3 matches, 
5 insertions and another 2 matches.
 
NOTE: only deal with Match, Gap, Soft Clip, Insertion, Deletion


This python3 module was converted from python2.7 code using 2to3

'''


__author__ = "Liguo Wang"
__copyright__ = "Copyleft"
__credits__ = []
__license__ = "GPL"
__version__="3.0.0"
__maintainer__ = "Liguo Wang"
__email__ = "wang.liguo@mayo.edu"
__status__ = "Production"


def map_bounds(start_pos, cigar):
    '''return the start, end corrdinates (on genome) of mapped region'''
    span_size = 0   #the actual genomic region size that covered by read
    for c,s in cigar:   #code and size
        if c == 0:  #match
            span_size += s
        elif c == 1:    #insertion to the ref
            continue
        elif c == 2: # deleltion
            span_size += s
        elif c == 3:    # skipping
            span_size += s
        else:
            continue
    return (start_pos, start_pos+span_size)
        

def fetch_exon(chrom, st, cigar):
    ''' fetch exon regions defined by cigar. st must be zero based
    return list of tuple of (chrom,st, end)
    '''
    #match = re.compile(r'(\d+)(\D)')
    chrom_st = st
    exon_bound =[]
    for c,s in cigar:   #code and size
        if c==0:        #match
            exon_bound.append((chrom, chrom_st,chrom_st + s))
            chrom_st += s
        elif c==1:      #insertion to ref
            continue
        elif c==2:      #deletion to ref
            chrom_st += s
        elif c==3:      #gap or intron
            chrom_st += s
        elif c==4:      #soft clipping. We do NOT include soft clip as part of exon
            chrom_st += s
        else:
            continue
    return exon_bound
    
def fetch_intron(chrom, st, cigar):
    ''' fetch intron regions defined by cigar. st must be zero based
    return list of tuple of (chrom,st, end)
    '''
    #match = re.compile(r'(\d+)(\D)')
    chrom_st = st
    intron_bound =[]
    for c,s in cigar:   #code and size
        if c==0:        #match
            chrom_st += s
        elif c==1:      #insertion to ref
            continue
        elif c==2:      #deletion to ref
            chrom_st += s
        elif c==3:      #gap or intron
            intron_bound.append((chrom, chrom_st,chrom_st+s))
            chrom_st += s
        elif c==4:      #soft clipping. We do NOT include soft clip as part of exon
            #chrom_st += s
            continue
        else:
            continue
    return intron_bound 

def fetch_clip(chrom, st, cigar):
    ''' fetch head soft clip regions defined by cigar. st must be zero based
    return list of tuple of (chrom,st, end)
    '''
    #match = re.compile(r'(\d+)(\D)')
    chrom_st = st
    clip_bound =[]
    for c,s in cigar:   #code and size
        if c==0:        #match
            chrom_st += s
        elif c==1:      #insertion to ref
            continue
        elif c==2:      #deletion to ref
            chrom_st += s
        elif c==3:      #gap or intron
            chrom_st += s
        elif c==4:      #soft clipping. We do NOT include soft clip as part of exon
            clip_bound.append((chrom, chrom_st, chrom_st + s))
            chrom_st += s
        else:
            continue
    return clip_bound       

def fetch_deletion(chrom, st, cigar):
    ''' fetch deletion regions defined by cigar. st must be zero based
    return list of tuple of (chrom,st, end)
    '''
    #match = re.compile(r'(\d+)(\D)')
    chrom_st = st
    del_bound =[]
    for c,s in cigar:   #code and size
        if c==0:        #match
            chrom_st += s
        elif c==1:      #insertion to ref
            continue
        elif c==2:      #deletion to ref
            del_bound.append((chrom, chrom_st, chrom_st + s))
            chrom_st += s
        elif c==3:      #gap or intron
            chrom_st += s
        elif c==4:      #soft clipping. We do NOT include soft clip as part of exon
            chrom_st += s
        else:
            continue
    return del_bound            

def fetch_deletion_range(cigar):
    ''' fetch deletion regions defined by cigar. st must be zero based
    return list of tuple of (st, end). 'st','end' is relative to the 
    start of read.
    '''
    del_bound =[]
    st = 0
    for c,s in cigar:   #code and size
        if c==0:        #match
            st += s
        elif c==4:
            st += s     #soft clip
        elif c==1:      #insertion to ref
            st += s
        elif c==2:      #deletion to ref
            del_bound.append((st,s))    #only record the start position of deletion, and the deletion size
        elif c==3:      #gap or intron
            continue
        else:
            continue
    return del_bound


def fetch_insertion_range(cigar):
    ''' fetch insertion regions defined by cigar. st must be zero based
    return list of tuple of (st, end). 'st','end' is relative to the 
    start of read.
    '''
    ins_bound =[]
    st = 0
    for c,s in cigar:   #code and size
        if c==0:        #match
            st += s
        elif c==4:
            st += s     #soft clip
        elif c==1:      #insertion to ref
            ins_bound.append((st, s))
            st += s
        elif c==2:      #deletion to ref
            continue
        elif c==3:      #gap or intron
            continue
        else:
            continue
    return ins_bound


    
def fetch_insertion(chrom, st, cigar):
    ''' fetch insertion regions defined by cigar. st must be zero based
    return list of tuple of (chrom,st, end)
    
    NOTE: insertion region does NOT present in reference genome and there
    fore cannot represented using reference coordinates[start, end]. So we
    use [start, SIZE).
    [(100,2)] means 2nt insert between 100 and 101
    '''
    #match = re.compile(r'(\d+)(\D)')
    chrom_st = st
    ins_bound =[]
    for c,s in cigar:   #code and size
        if c==0:        #match
            chrom_st += s
        elif c==1:      #insertion to ref
            ins_bound.append((chrom, chrom_st, s))
            continue
        elif c==2:      #deletion to ref
            chrom_st += s
        elif c==3:      #gap or intron
            chrom_st += s
        elif c==4:      #soft clipping. We do NOT include soft clip as part of exon
            chrom_st += s
        else:
            continue
    return ins_bound            


def list2str (lst):
    '''translate samtools returned cigar_list into cigar_string
    
    [(4, 1), (0, 9)] ==> '1S9M'
    '''
    code2Char={'0':'M','1':'I','2':'D','3':'N','4':'S','5':'H','6':'P','7':'=','8':'X'}
        
    cigar_str=''
    for i in lst:
        cigar_str += str(i[1]) + code2Char[str(i[0])]
    return cigar_str

def list2longstr (lst):
    '''translate samtools returned cigar_list into LONG cigar_string
    Sum of lengths of the M/I/S/=/X operations shall equal the length of SEQ
    '''
    
    code2Char={'0':'M','1':'I','2':'D','3':'N','4':'S','5':'H','6':'P','7':'=','8':'X'}
        
    cigar_str=''
    for i in lst:
        (code,length) = (i[0],i[1])
        if code in (0,1,4,7,8):
            cigar_str += code2Char[str(code)] * int(length)
        else:
            continue
    return cigar_str

    



#!/usr/bin/env python

'''
calculate raw read count, FPM (fragment per million) and FPKM (fragment per million mapped
reads per kilobase exon) for each gene in BED file.
'''

#import built-in modules
import os,sys
if sys.version_info[0] != 3:
	print("\nYou are using python" + str(sys.version_info[0]) + '.' + str(sys.version_info[1]) + " This verion of RSeQC needs python3!\n", file=sys.stderr)
	sys.exit()
	

import re
import string
from optparse import OptionParser
import warnings
import string
import collections
import math
from time import strftime

#import third-party modules
from bx.bitset import *
from bx.bitset_builders import *
from bx.intervals import *

#import my own modules
from qcmodule import SAM
from qcmodule import bam_cigar
#changes to the paths

#changing history to this module


__author__ = "Liguo Wang"
__copyright__ = "Copyleft"
__credits__ = []
__license__ = "GPL"
__version__="3.0.0"
__maintainer__ = "Liguo Wang"
__email__ = "wang.liguo@mayo.edu"
__status__ = "Production"


def printlog (mesg):
	'''print progress into stderr and log file'''
	mesg="@ " + strftime("%Y-%m-%d %H:%M:%S") + ": " + mesg
	LOG=open('class.log','a')
	print(mesg, file=sys.stderr)
	print(mesg, file=LOG)

def load_chromsize(file):
        '''read chrom.size file'''
        chromSize={}
        for line in open(file,'r'):
                if line.startswith('#'):continue
                if not line.strip():continue
                fields = line.strip().split()
                chromSize[fields[0]] = int(fields[1])
        return chromSize

def build_range(refgene):
	'''build ranges for exonic region'''
	ranges={}
	for line in open(refgene,'r'):
		try:
			if line.startswith(('#','track','browser')):continue

			# Parse fields from gene tabls
			fields = line.split()
			chrom     = fields[0].upper()
			tx_start  = int( fields[1] )
			tx_end    = int( fields[2] )
			geneName      = fields[3]
			strand    = fields[5].replace(" ","_")
				
			exon_starts = list(map( int, fields[11].rstrip( ',\n' ).split( ',' ) ))
			exon_starts = list(map((lambda x: x + tx_start ), exon_starts))
			exon_ends = list(map( int, fields[10].rstrip( ',\n' ).split( ',' ) ))
			exon_ends = list(map((lambda x, y: x + y ), exon_starts, exon_ends));   
		except:
			print("[NOTE:input bed must be 12-column] skipped this line: " + line, end=' ', file=sys.stderr)
			continue	

		for st,end in zip(exon_starts,exon_ends):	
			if chrom not in ranges:
				ranges[chrom] = Intersecter()
			ranges[chrom].add_interval( Interval( st, end ) )
	return ranges
	

def main():
	usage="%prog [options]" + '\n' + __doc__ + "\n"
	parser = OptionParser(usage,version="%prog " + __version__)
	parser.add_option("-i","--input-file",action="store",type="string",dest="input_file",help="Alignment file in BAM format (SAM is not supported). [required]")
	parser.add_option("-o","--out-prefix",action="store",type="string",dest="output_prefix",help="Prefix of output files(s). [required]")
	parser.add_option("-r","--refgene",action="store",type="string",dest="refgene_bed",help="Reference gene model in bed fomat. [required]")
	parser.add_option("-d","--strand",action="store",type="string",dest="strand_rule",default=None,help="How read(s) were stranded during sequencing. For example: --strand='1++,1--,2+-,2-+' means that this is a pair-end, strand-specific RNA-seq, and the strand rule is: read1 mapped to '+' => parental gene on '+'; read1 mapped to '-' => parental gene on '-'; read2 mapped to '+' => parental gene on '-'; read2 mapped to '-' => parental gene on '+'.  If you are not sure about the strand rule, run \'infer_experiment.py' default=%default (Not a strand specific RNA-seq data)")
	parser.add_option("-u","--skip-multi-hits",action="store_true",dest="skip_multi",help="How to deal with multiple hit reads. Presence this option renders program to skip multiple hits reads.")
	parser.add_option("-e","--only-exonic",action="store_true",dest="only_exon",help="How to count total reads. Presence of this option renders program only used exonic (UTR exons and CDS exons) reads, otherwise use all reads.")
	parser.add_option("-q","--mapq",action="store",type="int",dest="map_qual",default=30,help="Minimum mapping quality (phred scaled) for an alignment to be called \"uniquely mapped\". default=%default")
	parser.add_option("-s","--single-read",action="store",type="float",dest="single_read",default=1,help="How to count read-pairs that only have one end mapped. 0: ignore it. 0.5: treat it as half fragment. 1: treat it as whole fragment. default=%default")
	
	(options,args)=parser.parse_args()

	if not (options.output_prefix and options.input_file and options.refgene_bed):
		parser.print_help()
		sys.exit(0)
	if not os.path.exists(options.input_file + '.bai'):
		print("cannot find index file of input BAM file", file=sys.stderr)
		print(options.input_file + '.bai' + " does not exists", file=sys.stderr)
		sys.exit(0)
	for file in (options.input_file, options.refgene_bed):
		if not os.path.exists(file):
			print(file + " does NOT exists" + '\n', file=sys.stderr)
			sys.exit(0)

	obj = SAM.ParseBAM(options.input_file)
	OUT = open(options.output_prefix + '.FPKM.xls','w')

	#++++++++++++++++++++++++++++++++++++determine strand rule
	strandRule={}
	if options.strand_rule is None:													# Not strand-specific
		pass																
	elif len(options.strand_rule.split(',')) ==4:									#PairEnd, strand-specific
		for i in options.strand_rule.split(','):strandRule[i[0]+i[1]]=i[2]
	elif len(options.strand_rule.split(',')) ==2:									#singeEnd, strand-specific
		for i in options.strand_rule.split(','):strandRule[i[0]]=i[1]
	else:
		print("Unknown value of option :'strand_rule' " +  options.strand_rule, file=sys.stderr)
		sys.exit(1)	

	
	#++++++++++++++++++++++++++++++++++++counting fragments
	print("Extract exon regions from  "+ options.refgene_bed + '...', file=sys.stderr)
	gene_ranges = build_range( options.refgene_bed)
	print("Counting total fragment ... ", end=' ', file=sys.stderr)
	
	total_frags =0.0
	exonic_frags = 0.0
	
	try:
		while(1):
			aligned_read = next(obj.samfile)
			if aligned_read.is_qcfail:continue			#skip low quanlity					
			if aligned_read.is_duplicate:continue		#skip duplicate read
			if aligned_read.is_secondary:continue		#skip non primary hit
			if options.skip_multi:
				if aligned_read.mapq < options.map_qual:
					continue
			try:
				chrom = obj.samfile.getrname(aligned_read.tid).upper()
			except:
				continue
			read_st = aligned_read.pos
			read_end = read_st + aligned_read.rlen 	#not exactly the end position in case of splicing, insertion,etc

			if not aligned_read.is_paired:	# if read is NOT paired in sequencing (single-end sequencing)
				total_frags += 1
				if (chrom in gene_ranges) and len(gene_ranges[chrom].find(read_st, read_end)) >0:
					exonic_frags += 1
			elif aligned_read.is_paired:	# for pair-end sequencing
				if aligned_read.is_read2: continue	# only count read1
				mate_st = aligned_read.pnext
				mate_end = mate_st + aligned_read.rlen
				
				if aligned_read.is_unmapped:	#read1 unmapped
					if aligned_read.mate_is_unmapped: continue	#both unmap
					else:	#read2 is mapped
						total_frags += options.single_read
						if (chrom in gene_ranges)  and (len(gene_ranges[chrom].find(mate_st, mate_end)) >0 ):
							exonic_frags += options.single_read
				else:
					if aligned_read.mate_is_unmapped:
						total_frags += options.single_read
						if (chrom in gene_ranges) and (len(gene_ranges[chrom].find(read_st, read_end)) >0 ):
							exonic_frags += options.single_read
					else:
						total_frags += 1
						if (chrom in gene_ranges) and (len(gene_ranges[chrom].find(read_st, read_end)) >0 ) and (len(gene_ranges[chrom].find(mate_st, mate_end)) >0 ):
							exonic_frags += 1
					
	except StopIteration:
		print("Done", file=sys.stderr)
	print("Total fragment = %-20s" % (str(total_frags)), file=sys.stderr)
	print("Total exonic fragment = %-20s" % (str(exonic_frags)), file=sys.stderr)

	if total_frags >0 and exonic_frags > 0:
		if options.only_exon:
			denominator = exonic_frags
		else:
			denominator = total_frags
	else:
		print("Total tags cannot be 0 or negative number", file=sys.stderr)
		sys.exit(1)
	
	
	#++++++++++++++++++++++++++++++++++++++++++++++++
	obj = SAM.ParseBAM(options.input_file)
	print('\t'.join(('#chrom','st','end','accession','mRNA_size','gene_strand','Frag_count','FPM','FPKM')), file=OUT)

	gene_finished=0

	#calculate raw count, FPM, FPKM for each gene
	for line in open(options.refgene_bed,'r'):
		frag_count_f = 0.0
		frag_count_r = 0.0
		frag_count_fr = 0.0
		mRNA_size = 0.0
		exon_ranges= Intersecter()
		if line.startswith(('#','track','browser')):continue   
		fields = line.split()
		chrom     = fields[0]
		tx_start  = int( fields[1] )
		tx_end    = int( fields[2] )
		geneName      = fields[3]
		gstrand    = fields[5].replace(" ","_")
	    	
		exon_starts = list(map( int, fields[11].rstrip( ',\n' ).split( ',' ) ))
		exon_starts = list(map((lambda x: x + tx_start ), exon_starts))
		exon_ends = list(map( int, fields[10].rstrip( ',\n' ).split( ',' ) ))
		exon_ends = list(map((lambda x, y: x + y ), exon_starts, exon_ends));   
		for st,end in zip(exon_starts,exon_ends):
			mRNA_size += (end - st)
			exon_ranges.add_interval( Interval( st, end ) )
		
		
		# extract reads mapped gene region
		try:
			alignedReads = obj.samfile.fetch(chrom,tx_start,tx_end)
		except:
			continue
		for aligned_read in alignedReads:
			flag=0
			if aligned_read.is_qcfail:continue			#skip low quanlity					
			if aligned_read.is_duplicate:continue		#skip duplicate read
			if aligned_read.is_secondary:continue		#skip non primary hit		
			if options.skip_multi:
				if aligned_read.mapq < options.map_qual:
					continue
			
			#single end sequencing
			if not aligned_read.is_paired:
				frag_st = aligned_read.pos
				frag_end = frag_st + aligned_read.rlen 	#not exactly the end position in case of splicing, insertion,etc
				if aligned_read.is_reverse:
					strand_key = '-'
				else:
					strand_key = '+'
				
				if len(exon_ranges.find(frag_st, frag_end)) > 0:
					if options.strand_rule is None:
						frag_count_fr += 1
					elif strand_key in strandRule and strandRule[strand_key] == '+':
						frag_count_f += 1
					elif strand_key in strandRule and strandRule[strand_key] == '-':
						frag_count_r += 1
			
			# pair-end sequencing
			if aligned_read.is_paired:
				frag_st = aligned_read.pos
				frag_end = aligned_read.pnext
				if len(exon_ranges.find(frag_st, frag_st +1 ))  < 1 and len(exon_ranges.find(frag_end, frag_end +1 )) < 1:
					continue
				if aligned_read.is_read2:
					continue
				if aligned_read.is_reverse:
					strand_key = '1-'
				else:
					strand_key = '1+'								
				
				if options.strand_rule is None:
					if aligned_read.is_unmapped:
						if aligned_read.mate_is_unmapped:	# both unmapped
							continue
						else:	#only read2 mapped
							frag_count_fr += options.single_read
					else:
						if aligned_read.mate_is_unmapped:	# only read1 mapped
							frag_count_fr += options.single_read
						else:	#both mapped
							frag_count_fr += 1
				else:
					if strand_key in strandRule and strandRule[strand_key] == '+':
						if aligned_read.is_unmapped:
							if aligned_read.mate_is_unmapped:	# both unmapped
								continue
							else:	#only read2 mapped
								frag_count_f += options.single_read
						else:
							if aligned_read.mate_is_unmapped:	# only read1 mapped
								frag_count_f += options.single_read
							else:	#both mapped
								frag_count_f += 1
					if strand_key in strandRule and strandRule[strand_key] == '-':
						if aligned_read.is_unmapped:
							if aligned_read.mate_is_unmapped:	# both unmapped
								continue
							else:	#only read2 mapped
								frag_count_r += options.single_read
						else:
							if aligned_read.mate_is_unmapped:	# only read1 mapped
								frag_count_r += options.single_read
							else:	#both mapped
								frag_count_r += 1
		
		FPM_fr =  frag_count_fr * 1000000 / denominator
		FPM_f =  frag_count_f * 1000000 / denominator
		FPM_r =  frag_count_r * 1000000 / denominator
		FPKM_fr =  frag_count_fr * 1000000000 / (denominator * mRNA_size)
		FPKM_f =  frag_count_f * 1000000000 / (denominator * mRNA_size)
		FPKM_r =  frag_count_r * 1000000000 / (denominator * mRNA_size)
		
		
		if options.strand_rule is None:
			print('\t'.join([str(i) for i in (chrom, tx_start, tx_end, geneName, mRNA_size, gstrand, frag_count_fr, FPM_fr,FPKM_fr)]), file=OUT)
		else:
			if gstrand == '+':
				print('\t'.join([str(i) for i in (chrom, tx_start, tx_end, geneName, mRNA_size, gstrand, frag_count_f, FPM_f, FPKM_f )]), file=OUT)
			elif gstrand == '-':
				print('\t'.join([str(i) for i in (chrom, tx_start, tx_end, geneName, mRNA_size, gstrand, frag_count_r, FPM_r, FPKM_r)]), file=OUT)

		gene_finished +=1
		print(" %d transcripts finished\r" % (gene_finished), end=' ', file=sys.stderr)

if __name__ == '__main__':
	main()



#!/usr/bin/env python
'''
Check reads distribution over exon, intron, UTR, intergenic ... etc
The following reads will be skipped:
	qc_failed
	PCR duplicate
	Unmapped
	Non-primary (or secondary)	
'''

#import built-in modules
import os,sys
if sys.version_info[0] != 3:
	print("\nYou are using python" + str(sys.version_info[0]) + '.' + str(sys.version_info[1]) + " This verion of RSeQC needs python3!\n", file=sys.stderr)
	sys.exit()	


import re
import string
from optparse import OptionParser
import warnings
import collections
import math

#import third-party modules
from bx.bitset import *
from bx.bitset_builders import *
from bx.intervals import *
from bx.binned_array import BinnedArray
from bx_extras.fpconst import isNaN
from bx.bitset_utils import *

#import my own modules
from qcmodule import BED
from qcmodule import SAM
from qcmodule import bam_cigar

__author__ = "Liguo Wang"
__copyright__ = "Copyleft"
__credits__ = []
__license__ = "GPL"
__version__="3.0.0"
__maintainer__ = "Liguo Wang"
__email__ = "wang.liguo@mayo.edu"
__status__ = "Production"

def cal_size(list):
	'''calcualte bed list total size'''
	size=0
	for l in list:
		size += l[2] - l[1]
	return size

def foundone(chrom,ranges, st, end):
	found = 0
	if chrom in ranges:
		found = len(ranges[chrom].find(st,end))
	return found

def build_bitsets(list):
	'''build intevalTree from list'''
	ranges={}
	for l in list:
		chrom =l[0].upper()
		st = int(l[1])
		end = int(l[2])
		if chrom not in ranges:
			ranges[chrom] = Intersecter()
		ranges[chrom].add_interval( Interval( st, end ) )
	return ranges

def process_gene_model(gene_model):
	print("processing " + gene_model + ' ...', end=' ', file=sys.stderr)
	obj = BED.ParseBED(gene_model)
	utr_3 = obj.getUTR(utr=3)
	utr_5 = obj.getUTR(utr=5)
	cds_exon = obj.getCDSExon()
	intron = obj.getIntron()
	
	intron = BED.unionBed3(intron)
	cds_exon=BED.unionBed3(cds_exon)
	utr_5 = BED.unionBed3(utr_5)
	utr_3 = BED.unionBed3(utr_3)
	
	utr_5 = BED.subtractBed3(utr_5,cds_exon)
	utr_3 = BED.subtractBed3(utr_3,cds_exon)
	intron = BED.subtractBed3(intron,cds_exon)
	intron = BED.subtractBed3(intron,utr_5)
	intron = BED.subtractBed3(intron,utr_3)
	
	intergenic_up_1kb = obj.getIntergenic(direction="up",size=1000)
	intergenic_down_1kb = obj.getIntergenic(direction="down",size=1000)
	intergenic_up_5kb = obj.getIntergenic(direction="up",size=5000)
	intergenic_down_5kb = obj.getIntergenic(direction="down",size=5000)	
	intergenic_up_10kb = obj.getIntergenic(direction="up",size=10000)
	intergenic_down_10kb = obj.getIntergenic(direction="down",size=10000)
	
	#merge integenic region
	intergenic_up_1kb=BED.unionBed3(intergenic_up_1kb)
	intergenic_up_5kb=BED.unionBed3(intergenic_up_5kb)
	intergenic_up_10kb=BED.unionBed3(intergenic_up_10kb)
	intergenic_down_1kb=BED.unionBed3(intergenic_down_1kb)
	intergenic_down_5kb=BED.unionBed3(intergenic_down_5kb)
	intergenic_down_10kb=BED.unionBed3(intergenic_down_10kb)	
	
	#purify intergenic region
	intergenic_up_1kb=BED.subtractBed3(intergenic_up_1kb,cds_exon)
	intergenic_up_1kb=BED.subtractBed3(intergenic_up_1kb,utr_5)
	intergenic_up_1kb=BED.subtractBed3(intergenic_up_1kb,utr_3)
	intergenic_up_1kb=BED.subtractBed3(intergenic_up_1kb,intron)
	intergenic_down_1kb=BED.subtractBed3(intergenic_down_1kb,cds_exon)
	intergenic_down_1kb=BED.subtractBed3(intergenic_down_1kb,utr_5)
	intergenic_down_1kb=BED.subtractBed3(intergenic_down_1kb,utr_3)
	intergenic_down_1kb=BED.subtractBed3(intergenic_down_1kb,intron)	

	#purify intergenic region
	intergenic_up_5kb=BED.subtractBed3(intergenic_up_5kb,cds_exon)
	intergenic_up_5kb=BED.subtractBed3(intergenic_up_5kb,utr_5)
	intergenic_up_5kb=BED.subtractBed3(intergenic_up_5kb,utr_3)
	intergenic_up_5kb=BED.subtractBed3(intergenic_up_5kb,intron)
	intergenic_down_5kb=BED.subtractBed3(intergenic_down_5kb,cds_exon)
	intergenic_down_5kb=BED.subtractBed3(intergenic_down_5kb,utr_5)
	intergenic_down_5kb=BED.subtractBed3(intergenic_down_5kb,utr_3)
	intergenic_down_5kb=BED.subtractBed3(intergenic_down_5kb,intron)	
	
	#purify intergenic region
	intergenic_up_10kb=BED.subtractBed3(intergenic_up_10kb,cds_exon)
	intergenic_up_10kb=BED.subtractBed3(intergenic_up_10kb,utr_5)
	intergenic_up_10kb=BED.subtractBed3(intergenic_up_10kb,utr_3)
	intergenic_up_10kb=BED.subtractBed3(intergenic_up_10kb,intron)
	intergenic_down_10kb=BED.subtractBed3(intergenic_down_10kb,cds_exon)
	intergenic_down_10kb=BED.subtractBed3(intergenic_down_10kb,utr_5)
	intergenic_down_10kb=BED.subtractBed3(intergenic_down_10kb,utr_3)
	intergenic_down_10kb=BED.subtractBed3(intergenic_down_10kb,intron)	
	
	#build intervalTree 
	cds_exon_ranges = build_bitsets(cds_exon)
	utr_5_ranges = build_bitsets(utr_5)
	utr_3_ranges = build_bitsets(utr_3)
	intron_ranges = build_bitsets(intron)
	interg_ranges_up_1kb_ranges = build_bitsets(intergenic_up_1kb)
	interg_ranges_up_5kb_ranges = build_bitsets(intergenic_up_5kb)
	interg_ranges_up_10kb_ranges = build_bitsets(intergenic_up_10kb)
	interg_ranges_down_1kb_ranges = build_bitsets(intergenic_down_1kb)
	interg_ranges_down_5kb_ranges = build_bitsets(intergenic_down_5kb)
	interg_ranges_down_10kb_ranges = build_bitsets(intergenic_down_10kb)
	
	exon_size = cal_size(cds_exon)
	intron_size = cal_size(intron)
	utr3_size = cal_size(utr_3)
	utr5_size = cal_size(utr_5)
	int_up1k_size = cal_size(intergenic_up_1kb)
	int_up5k_size = cal_size(intergenic_up_5kb)
	int_up10k_size = cal_size(intergenic_up_10kb)
	int_down1k_size = cal_size(intergenic_down_1kb)
	int_down5k_size = cal_size(intergenic_down_5kb)
	int_down10k_size = cal_size(intergenic_down_10kb)
	
	print("Done", file=sys.stderr)
	return (cds_exon_ranges,intron_ranges,utr_5_ranges,utr_3_ranges,\
			interg_ranges_up_1kb_ranges,interg_ranges_up_5kb_ranges,interg_ranges_up_10kb_ranges,\
			interg_ranges_down_1kb_ranges,interg_ranges_down_5kb_ranges,interg_ranges_down_10kb_ranges,\
			exon_size,intron_size,utr5_size,utr3_size,\
			int_up1k_size,int_up5k_size,int_up10k_size,\
			int_down1k_size,int_down5k_size,int_down10k_size)
	
def main():
	usage="%prog [options]" + '\n' + __doc__ + "\n"
	parser = OptionParser(usage,version="%prog " + __version__)
	parser.add_option("-i","--input-file",action="store",type="string",dest="input_file",help="Alignment file in BAM or SAM format.")
	parser.add_option("-r","--refgene",action="store",type="string",dest="ref_gene_model",help="Reference gene model in bed format.")
	(options,args)=parser.parse_args()
		
	if not (options.input_file and options.ref_gene_model):
		parser.print_help()
		sys.exit(0)
	if not os.path.exists(options.ref_gene_model):
		print('\n\n' + options.ref_gene_model + " does NOT exists" + '\n', file=sys.stderr)
		#parser.print_help()
		sys.exit(0)
	if not os.path.exists(options.input_file):
		print('\n\n' + options.input_file + " does NOT exists" + '\n', file=sys.stderr)
		sys.exit(0)		

	#build bitset
	(cds_exon_r, intron_r, utr_5_r, utr_3_r,\
	intergenic_up_1kb_r,intergenic_up_5kb_r,intergenic_up_10kb_r,\
	intergenic_down_1kb_r,intergenic_down_5kb_r,intergenic_down_10kb_r,\
	cds_exon_base,intron_base,utr_5_base,utr_3_base,\
	intergenic_up1kb_base,intergenic_up5kb_base,intergenic_up10kb_base,\
	intergenic_down1kb_base,intergenic_down5kb_base,intergenic_down10kb_base) = process_gene_model(options.ref_gene_model)
	
	intron_read=0
	cds_exon_read=0
	utr_5_read=0
	utr_3_read=0
	
	intergenic_up1kb_read=0
	intergenic_down1kb_read=0
	intergenic_up5kb_read=0
	intergenic_down5kb_read=0
	intergenic_up10kb_read=0
	intergenic_down10kb_read=0
		
	totalReads=0
	totalFrags=0
	unAssignFrags=0
	obj = SAM.ParseBAM(options.input_file)
	
	R_qc_fail=0
	R_duplicate=0
	R_nonprimary=0
	R_unmap=0
	
	print("processing " + options.input_file + " ...", end=' ', file=sys.stderr)
	try:
		while(1):
			aligned_read = next(obj.samfile)
			if aligned_read.is_qcfail:			#skip QC fail read
				R_qc_fail +=1
				continue
			if aligned_read.is_duplicate:		#skip duplicate read
				R_duplicate +=1
				continue
			if aligned_read.is_secondary:		#skip non primary hit
				R_nonprimary +=1
				continue
			if aligned_read.is_unmapped:		#skip unmap read
				R_unmap +=1
				continue		
			totalReads +=1
			chrom = obj.samfile.getrname(aligned_read.tid)
			chrom=chrom.upper()
			exons = bam_cigar.fetch_exon(chrom, aligned_read.pos, aligned_read.cigar)
			totalFrags += len(exons)
			
			for exn in exons:
				#print chrom + '\t' + str(exn[1]) + '\t' + str(exn[2])
				mid = int(exn[1]) + int((int(exn[2]) - int(exn[1]))/2)
				if foundone(chrom,cds_exon_r,mid,mid) > 0:
					cds_exon_read += 1
					continue
				elif foundone(chrom,utr_5_r,mid,mid) >0 and foundone(chrom,utr_3_r,mid,mid) == 0:
					utr_5_read += 1
					continue
				elif foundone(chrom,utr_3_r,mid,mid) >0 and foundone(chrom,utr_5_r,mid,mid) == 0:
					utr_3_read += 1
					continue
				elif foundone(chrom,utr_3_r,mid,mid) >0 and foundone(chrom,utr_5_r,mid,mid) > 0:
					unAssignFrags +=1
					continue
				elif foundone(chrom,intron_r,mid,mid) > 0:
					intron_read += 1
					continue
				elif foundone(chrom,intergenic_up_10kb_r,mid,mid) >0 and foundone(chrom,intergenic_down_10kb_r,mid,mid) > 0:
					unAssignFrags +=1
					continue					
				elif foundone(chrom,intergenic_up_1kb_r,mid,mid) >0:
					intergenic_up1kb_read += 1
					intergenic_up5kb_read += 1
					intergenic_up10kb_read += 1
				elif foundone(chrom,intergenic_up_5kb_r,mid,mid) >0:
					intergenic_up5kb_read += 1
					intergenic_up10kb_read += 1
				elif foundone(chrom,intergenic_up_10kb_r,mid,mid) >0:
					intergenic_up10kb_read += 1
				
				elif foundone(chrom,intergenic_down_1kb_r,mid,mid) >0:
					intergenic_down1kb_read += 1
					intergenic_down5kb_read += 1
					intergenic_down10kb_read += 1
				elif foundone(chrom,intergenic_down_5kb_r,mid,mid) >0:
					intergenic_down5kb_read += 1
					intergenic_down10kb_read += 1
				elif foundone(chrom,intergenic_down_10kb_r,mid,mid) >0:
					intergenic_down10kb_read += 1	
				else:
					unAssignFrags +=1
	except StopIteration:
		print("Finished\n", file=sys.stderr)				

	print("%-30s%d" % ("Total Reads",totalReads))
	print("%-30s%d" % ("Total Tags",totalFrags))
	print("%-30s%d" % ("Total Assigned Tags",totalFrags-unAssignFrags))
	
	print("=====================================================================")
	print("%-20s%-20s%-20s%-20s" % ('Group','Total_bases','Tag_count','Tags/Kb'))
	print("%-20s%-20d%-20d%-18.2f" % ('CDS_Exons',cds_exon_base,cds_exon_read,cds_exon_read*1000.0/(cds_exon_base+1)))
	print("%-20s%-20d%-20d%-18.2f" % ("5'UTR_Exons",utr_5_base,utr_5_read, utr_5_read*1000.0/(utr_5_base+1)))
	print("%-20s%-20d%-20d%-18.2f" % ("3'UTR_Exons",utr_3_base,utr_3_read, utr_3_read*1000.0/(utr_3_base+1)))
	print("%-20s%-20d%-20d%-18.2f" % ("Introns",intron_base,intron_read,intron_read*1000.0/(intron_base+1)))
	
	print("%-20s%-20d%-20d%-18.2f" % ("TSS_up_1kb",intergenic_up1kb_base, intergenic_up1kb_read, intergenic_up1kb_read*1000.0/(intergenic_up1kb_base+1)))
	print("%-20s%-20d%-20d%-18.2f" % ("TSS_up_5kb",intergenic_up5kb_base, intergenic_up5kb_read, intergenic_up5kb_read*1000.0/(intergenic_up5kb_base+1)))
	print("%-20s%-20d%-20d%-18.2f" % ("TSS_up_10kb",intergenic_up10kb_base, intergenic_up10kb_read, intergenic_up10kb_read*1000.0/(intergenic_up10kb_base+1)))
	print("%-20s%-20d%-20d%-18.2f" % ("TES_down_1kb",intergenic_down1kb_base, intergenic_down1kb_read, intergenic_down1kb_read*1000.0/(intergenic_down1kb_base+1)))
	print("%-20s%-20d%-20d%-18.2f" % ("TES_down_5kb",intergenic_down5kb_base, intergenic_down5kb_read, intergenic_down5kb_read*1000.0/(intergenic_down5kb_base+1)))	
	print("%-20s%-20d%-20d%-18.2f" % ("TES_down_10kb",intergenic_down10kb_base, intergenic_down10kb_read, intergenic_down10kb_read*1000.0/(intergenic_down10kb_base+1)))
	print("=====================================================================")
	
if __name__ == '__main__':
	main()



#!/usr/bin/env python
'''-------------------------------------------------------------------------------------------------
Split bam file according to input gene list (bed).
-------------------------------------------------------------------------------------------------'''

#import built-in modules
import os,sys

if sys.version_info[0] != 3:
	print("\nYou are using python" + str(sys.version_info[0]) + '.' + str(sys.version_info[1]) + " This verion of RSeQC needs python3!\n", file=sys.stderr)
	sys.exit()	

import string
from optparse import OptionParser
import warnings
import collections

#import third-party modules
from bx.bitset import *
from bx.bitset_builders import *
from bx.intervals import *
from bx.binned_array import BinnedArray
from bx_extras.fpconst import isNaN
from bx.bitset_utils import *
import pysam

#import my own modules
from qcmodule import BED
from qcmodule import SAM
from qcmodule import bam_cigar

__author__ = "Liguo Wang"
__copyright__ = "Copyleft"
__credits__ = []
__license__ = "GPL"
__version__="3.0.0"
__maintainer__ = "Liguo Wang"
__email__ = "wang.liguo@mayo.edu"
__status__ = "Production"

def searchit(exon_range, exon_list):
	'''return 1 if find, return 0 if cannot find'''
	for chrom, st, end in exon_list:
		if chrom.upper() not in exon_range:
			return 0
		elif len(exon_range[chrom].find(st,end)) >=1:
			return 1
	return 0

def build_bitsets(list):
	'''build intevalTree from list'''
	ranges={}
	for l in list:
		chrom =l[0].upper()
		st = int(l[1])
		end = int(l[2])
		if chrom not in ranges:
			ranges[chrom] = Intersecter()
		ranges[chrom].add_interval( Interval( st, end ) )
	return ranges

	
def main():
	usage="%prog [options]" + '\n' + __doc__ + "\n"
	parser = OptionParser(usage,version="%prog " + __version__)
	parser.add_option("-i","--input-file",action="store",type="string",dest="input_file",help="Alignment file in BAM or SAM format. BAM file should be sorted and indexed.")
	parser.add_option("-r","--genelist",action="store",type="string",dest="gene_list",help="Gene list in bed foramt. All reads hits to exon regions (defined by this gene list) will be saved into one BAM file, the remaining reads will saved into another BAM file.")
	parser.add_option("-o","--out-prefix",action="store",type="string",dest="output_prefix",help="Prefix of output BAM files. \"prefix.in.bam\" file contains reads mapped to the gene list specified by \"-r\", \"prefix.ex.bam\" contains reads that cannot mapped to gene list. \"prefix.junk.bam\" contains qcfailed or unmapped reads.")
	(options,args)=parser.parse_args()
		
	if not (options.input_file and options.gene_list):
		parser.print_help()
		sys.exit(0)
	if not os.path.exists(options.gene_list):
		print('\n\n' + options.gene_list + " does NOT exists" + '\n', file=sys.stderr)
		#parser.print_help()
		sys.exit(0)
	if not os.path.exists(options.input_file):
		print('\n\n' + options.input_file + " does NOT exists" + '\n', file=sys.stderr)
		sys.exit(0)		
	
	#build bitset for gene list
	print('reading ' + options.gene_list + ' ... ', end=' ', file=sys.stderr)
	obj = BED.ParseBED(options.gene_list)
	exons = obj.getExon()
	exon_ranges = build_bitsets(exons)
	print('Done', file=sys.stderr)
	
	samfile = pysam.Samfile(options.input_file,'rb')
	out1 = pysam.Samfile(options.output_prefix + '.in.bam','wb',template=samfile)	#bam file containing reads hit to exon region
	out2 = pysam.Samfile(options.output_prefix + '.ex.bam','wb',template=samfile)	#bam file containing reads not hit to exon region
	out3 = pysam.Samfile(options.output_prefix + '.junk.bam','wb',template=samfile)	#bam file containing reads not hit to exon region
	
	total_alignment = 0
	in_alignment = 0
	ex_alignment = 0
	bad_alignment = 0
	print("spliting " + options.input_file + " ...", end=' ', file=sys.stderr)
	try:
		while(1):
			aligned_read = next(samfile)
			total_alignment += 1
			
			if aligned_read.is_qcfail:
				bad_alignment +=1
				out3.write(aligned_read)
				continue
			if aligned_read.is_unmapped:
				bad_alignment +=1
				out3.write(aligned_read)
				continue
			
			chrom = samfile.getrname(aligned_read.tid)
			chrom=chrom.upper()	
			read_start = aligned_read.pos
			mate_start = aligned_read.mpos
				
			#read_exons = bam_cigar.fetch_exon(chrom, aligned_read.pos, aligned_read.cigar)
			if aligned_read.mate_is_unmapped:	#only one end mapped
				if chrom not in exon_ranges:
					out2.write(aligned_read)
					ex_alignment += 1
					continue		
				else:		
					if len(exon_ranges[chrom].find(read_start, read_start +1)) >= 1:
						out1.write(aligned_read)
						in_alignment += 1
						continue
					elif len(exon_ranges[chrom].find(read_start, read_start +1)) == 0:
						out2.write(aligned_read)
						ex_alignment += 1
						continue
			else:							#both end mapped
				if chrom not in exon_ranges:
					out2.write(aligned_read)
					ex_alignment += 1
					continue
				else:
					if (len(exon_ranges[chrom].find(read_start, read_start +1)) >= 1) or (len(exon_ranges[chrom].find(mate_start, mate_start +1)) >= 1):
						out1.write(aligned_read)
						in_alignment += 1
					else:
						out2.write(aligned_read)
						ex_alignment += 1
				
	except StopIteration:
		print("Done", file=sys.stderr)
				
	print("%-55s%d" % ("Total records:",total_alignment))
	print("%-55s%d" % (options.output_prefix + '.in.bam (Alignments consumed by input gene list):',in_alignment))
	print("%-55s%d" % (options.output_prefix + '.ex.bam (Alignments not consumed by input gene list):',ex_alignment))
	print("%-55s%d" % (options.output_prefix + '.junk.bam (qcfailed, unmapped reads):',bad_alignment))
	
if __name__ == '__main__':
	main()



'''
manipulate CIGAR string
This python3 module was converted from python2.7 code using 2to3
'''

#import built-in modules
import os,sys
import re
import string
import warnings
import string
import collections
import math
import random

__author__ = "Liguo Wang"
__copyright__ = "Copyleft"
__credits__ = []
__license__ = "GPL"
__version__="3.0.0"
__maintainer__ = "Liguo Wang"
__email__ = "wang.liguo@mayo.edu"
__status__ = "Production"

head_clip = re.compile(r'^(\d+)S')
tail_clip = re.compile(r'(\d+)S$')
insertion = re.compile(r'(\d+)I')
deletion  = re.compile(r'(\d+)D')
matching  = re.compile(r'(\d+)M')
skipping  = re.compile(r'(\d+)N')
read_part = re.compile(r'(\d+)[MIS=X]')
ref_part = re.compile(r'(\d+)[MISND=X]')

prior_insertion = re.compile(r'(.+?)(\d+)I')
prior_deletion = re.compile(r'(.+?)(\d+)D')
prior_intron = re.compile(r'(.+?)(\d+)N')
prior_exon = re.compile(r'(.*?)(\d+)M')

def fetch_head_clip(chr, st, cigar):
    '''return genome coordinate of the head clip part encoded in cigar string
    NOTE: returned coordinates are 0-based.NOTE: st is 0-based'''
    
    block=[]
    chrom_end = int(st)     
    tmp = head_clip.findall(cigar)
    if len(tmp)==0:
        return block
    else:
        chrom_st = chrom_end - int(tmp[0])
        block.append([chr,chrom_st,chrom_end])
    return block

def fetch_tail_clip(chr, st, cigar):
    '''return genome coordinates of the tail clip part encoded in cigar string
    NOTE: returned coordinates are 0-based .  NOTE: st is 0-based'''
    
    block=[]
    h = head_clip.findall(cigar)
    t = tail_clip.findall(cigar)
    if len(t)==0:return block
    else:t_len = int(t[0])
    
    if len(h)==0:h_len=0
    else:h_len=int(h[0])
    ref_length = sum([int(i) for i in ref_part.findall(cigar)])
    #print read_length
    chrom_end = int(st) + (ref_length - h_len)      #because SAM is 1-based
    chrom_st = chrom_end - t_len
    block.append([chr,chrom_st,chrom_end])
    return block
    
def fetch_insertion(chr, st, cigar):
    '''return genome coordinates of the insertion (to reference) encoded in cigar string
    NOTE: returned coordinates are 0-based. Insertion to the reference.  NOTE: st is 0-based'''
    
    block=[]
    h = head_clip.findall(cigar)
    if len(h)==0:h_len=0
    else:h_len=int(h[0])
    
    ref_length=0
    m = prior_insertion.findall(cigar)
    if len(m)==0:
        return block
    else:
        for j in m:
            ref_length += sum([int(i) for i in ref_part.findall(j[0])])
            chrom_st = int(st) + (ref_length - h_len)
            chrom_end = chrom_st + int(j[1])
            block.append([chr,chrom_st,chrom_end])
    return block

def fetch_deletion(chr, st, cigar):
    '''return genome coordinates of the insertion (to reference) encoded in cigar string
    NOTE: returned coordinates are 0-based. Deletion to the reference.  NOTE: st is 0-based'''
    
    block=[]
    h = head_clip.findall(cigar)
    if len(h)==0:h_len=0
    else:h_len=int(h[0])
    
    ref_length=0
    m = prior_deletion.findall(cigar)
    if len(m)==0:
        return block
    else:
        for j in m:
            ref_length += sum([int(i) for i in ref_part.findall(j[0])])
            chrom_st = int(st) + (ref_length - h_len)
            chrom_end = chrom_st + int(j[1])
            block.append([chr,chrom_st,chrom_end])
            ref_length += int(j[1])
    return block

    
def fetch_intron(chr, st, cigar):
    '''return genome coordinates of the introns encoded in cigar string
    NOTE: returned coordinates are 0-based. Deletion to the reference NOTE: 
    st is 0-based '''
    
    block=[]
    h = head_clip.findall(cigar)
    if len(h)==0:h_len=0
    else:h_len=int(h[0])
    
    ref_length=0
    m = prior_intron.findall(cigar)
    if len(m)==0:
        return block
    else:
        for j in m:
            ref_length += sum([int(i) for i in ref_part.findall(j[0])])
            chrom_st = int(st)  + (ref_length - h_len)
            chrom_end = chrom_st + int(j[1])
            block.append([chr,chrom_st,chrom_end])
            ref_length += int(j[1])
    return block
    
def fetch_exon(chr, st, cigar):
    '''return genome coordinates of the exon encoded in cigar string
    NOTE: returned coordinates are 0-based. NOTE: st is 0-based'''
    
    block=[]
    h = head_clip.findall(cigar)
    if len(h)==0:h_len=0
    else:h_len=int(h[0])
    
    ref_length=0
    m = prior_exon.findall(cigar)
    if len(m)==0:
        return block
    else:
        for j in m:
            ref_length += sum([int(i) for i in ref_part.findall(j[0])])
            chrom_st = int(st) + (ref_length - h_len)
            chrom_end = chrom_st + int(j[1])
            block.append([chr,chrom_st,chrom_end])
            ref_length += int(j[1])
    return block
    
def list2str (lst):
    '''translate samtools returned cigar_list into cigar_string'''
    code2Char={'0':'M','1':'I','2':'D','3':'N','4':'S','5':'H','6':'P','7':'=','8':'X'}
    
    cigar_str=''
    for i in lst:
        cigar_str += str(i[1]) + code2Char[str(i[0])]
    return cigar_str



#!/usr/bin/env python
'''
Description: Convert alignments in BAM or SAM format into fastq format.
'''

#import built-in modules
import os,sys
if sys.version_info[0] != 3:
	print("\nYou are using python" + str(sys.version_info[0]) + '.' + str(sys.version_info[1]) + " This verion of RSeQC needs python3!\n", file=sys.stderr)
	sys.exit()	

import re
import string
from optparse import OptionParser
import warnings
import string
import collections
import subprocess

from time import strftime

#import my own modules
from qcmodule import SAM
#changes to the paths

#changing history to this module


__author__ = "Liguo Wang"
__copyright__ = "Copyleft"
__credits__ = []
__license__ = "GPL"
__version__="3.0.0"
__maintainer__ = "Liguo Wang"
__email__ = "wang.liguo@mayo.edu"
__status__ = "Production"



def main():
	usage="%prog [options]" + '\n' + __doc__ + "\n"
	parser = OptionParser(usage,version="%prog " + __version__)
	parser.add_option("-i","--input-file",action="store",type="string",dest="input_file",help="Alignment file in BAM or SAM format.")
	parser.add_option("-o","--out-prefix",action="store",type="string",dest="output_prefix",help="Prefix of output fastq files(s).")
	parser.add_option("-s","--single-end",action="store_true",dest="single", help="Specificy '-s' or '--single-end' for single-end sequencing.")
	parser.add_option("-c","--compress",action="store_true",dest="gzip", help="Specificy '-c' or '--compress' to compress output fastq file(s) using 'gzip' command.")
	(options,args)=parser.parse_args()
	
	
	#print options.single
	if not (options.output_prefix and options.input_file):
		parser.print_help()
		sys.exit(0)
	if os.path.exists(options.input_file):
		obj = SAM.ParseBAM(options.input_file)
		if options.single is True:
			obj.bam2fq(prefix=options.output_prefix, paired = False)
			if options.gzip is True:
				try:
					print("run gzip ... ", end=' ', file=sys.stderr)
					subprocess.call("gzip " + options.output_prefix + '.fastq', shell=True)
					print("Done.", file=sys.stderr)
				except:
					pass
		else:
			obj.bam2fq(prefix=options.output_prefix, paired = True)
			if options.gzip is True:
				try:
					print("run gzip ...", file=sys.stderr)
					subprocess.call("gzip " + options.output_prefix + '.R1.fastq', shell=True)
					subprocess.call("gzip " + options.output_prefix + '.R2.fastq', shell=True)
					print("Done.", file=sys.stderr)
				except:
					pass
	else:
		print('\n\n' + options.input_file + " does NOT exists" + '\n', file=sys.stderr)
		#parser.print_help()
		sys.exit(0)

if __name__ == '__main__':
	main()



#!/usr/bin/env python
'''
Summarizing mapping statistics of a BAM or SAM file. 
'''

#import built-in modules
import os,sys
if sys.version_info[0] != 3:
	print("\nYou are using python" + str(sys.version_info[0]) + '.' + str(sys.version_info[1]) + " This verion of RSeQC needs python3!\n", file=sys.stderr)
	sys.exit()
	
import re
import string
from optparse import OptionParser
import warnings
import string
import collections
import math
from time import strftime

#import third-party modules
from bx.bitset import *
from bx.bitset_builders import *
from bx.intervals import *

#import my own modules
from qcmodule import SAM
#changes to the paths

#changing history to this module


__author__ = "Liguo Wang"
__copyright__ = "Copyleft."
__credits__ = []
__license__ = "GPL"
__version__="3.0.0"
__maintainer__ = "Liguo Wang"
__email__ = "wang.liguo@mayo.edu"
__status__ = "Production"


def main():
	usage="%prog [options]" + '\n' + __doc__ + "\n"
	parser = OptionParser(usage,version="%prog " + __version__)
	parser.add_option("-i","--input-file",action="store",type="string",dest="input_file",help="Alignment file in BAM or SAM format.")
	parser.add_option("-q","--mapq",action="store",type="int",dest="map_qual",default=30,help="Minimum mapping quality (phred scaled) to determine \"uniquely mapped\" reads. default=%default")
	(options,args)=parser.parse_args()

	if not (options.input_file):
		parser.print_help()
		sys.exit(0)
	if not os.path.exists(options.input_file):
		print('\n\n' + input_file + " does NOT exists" + '\n', file=sys.stderr)
		sys.exit(0)

	obj = SAM.ParseBAM(options.input_file)
	obj.stat(q_cut = options.map_qual)


if __name__ == '__main__':
	main()



#!/usr/bin/env python
'''deal with Kmer. DNA sequence should only A, C, G, T. python2.7 or newer'''

#import built-in modules
import os,sys
import numpy
import math
from collections import Counter
import re
import itertools

def word_generator(seq,word_size,step_size,frame=0):
	'''generate DNA word from sequence using word_size and step_size. Frame is 0, 1 or2'''
	for i in range(frame,len(seq),step_size):
		word =  seq[i:i+word_size]
		if len(word) == word_size:
			yield word

def seq_generator(fastafile):
	'''DNA sequence only contains A,C,G,T,N. sequence with other characters will be removed'''
	tmpseq=''
	name=''
	DNA_pat = re.compile(r'^[ACGTN]+$')
	for line in open(fastafile,'r'):
		line=line.strip().upper()
		if line.startswith(('#',' ','\n')):continue
		if line.startswith(('>','@')):
			if tmpseq:
				yield [name,tmpseq]
				tmpseq=''
			name = line.split()[0][1:]
		elif DNA_pat.match(line):
			tmpseq += line
	yield [name,tmpseq]
	
def all_possible_kmer(l):
	'''return all possible combinations of A,C,G,T,N. only support A,C,G,T,N. l is length of kmer'''
	for i in itertools.product(['A','C','G','T','N'],repeat=l):
		yield ''.join(i)

def kmer_freq_file (fastafile,word_size,step_size=1,frame=0,min_count=0):
	'''Calculate kmer frequency from fasta file'''
	seq_num = 0
	ret_dict={}
	for n,s in seq_generator(fastafile):
		seq_num += 1
		if seq_num == 1:
			count_table = Counter(word_generator(s,word_size=word_size,step_size=step_size,frame=frame))
		else:
			count_table.update( word_generator(s,word_size=word_size,step_size=step_size,frame=frame) )
	
	#return count_table
	for kmer in all_possible_kmer(word_size):
		if kmer not in count_table: count_table[kmer]=0
		if count_table[kmer] >= min_count:
			#print kmer + '\t' + str(count_table[kmer])
			if 'N' in kmer:continue
			ret_dict[kmer] = count_table[kmer]
	return ret_dict		

def kmer_freq_seq (seq,word_size,step_size=1,frame=0,min_count=0):
	'''Calculate kmer frequency from DNA sequence. coding. genome is hexamer table calculated
	from coding region and whole genome (as background control)
	'''
	count_table = Counter(word_generator(seq,word_size=word_size,step_size=step_size,frame=frame))
	for kmer in all_possible_kmer(word_size):
		if kmer not in count_table: count_table[kmer]=0
		if count_table[kmer] >= min_count:
			print(kmer + '\t' + str(count_table[kmer]))

def kmer_ratio(seq,word_size,step_size,coding,noncoding):
	if len(seq) < word_size:
		return 0
		
	sum_of_log_ratio_0 = 0.0
	sum_of_log_ratio_1 = 0.0
	sum_of_log_ratio_2 = 0.0	
	frame0_count=0.0
	frame1_count=0.0
	frame2_count=0.0
	for k in word_generator(seq=seq, word_size = word_size, step_size=step_size,frame=0):	
		if (k not in coding) or (k not in noncoding):
			continue
		if coding[k]>0 and noncoding[k] >0:
			sum_of_log_ratio_0  +=  math.log( coding[k] / noncoding[k])
		elif coding[k]>0 and noncoding[k] == 0:
			sum_of_log_ratio_0 += 1
		elif coding[k] == 0 and noncoding[k] == 0:
			continue
		elif coding[k] == 0 and noncoding[k] >0 :
			sum_of_log_ratio_0 -= 1
		else:
			continue
		frame0_count += 1
	'''	
	for k in word_generator(seq=seq, word_size = word_size, step_size=step_size,frame=1):
		if (not coding.has_key(k)) or (not noncoding.has_key(k)):
			continue
		if coding[k]>0 and noncoding[k] >0:
			sum_of_log_ratio_1  +=  math.log( coding[k] / noncoding[k])
		elif coding[k]>0 and noncoding[k] == 0:
			sum_of_log_ratio_1 += 1
		elif coding[k] == 0 and noncoding[k] == 0:
			continue
		elif coding[k] == 0 and noncoding[k] >0 :
			sum_of_log_ratio_1 -= 1
		else:
			continue
		frame1_count += 1
	
	for k in word_generator(seq=seq, word_size = word_size, step_size=step_size,frame=2):
		if (not coding.has_key(k)) or (not noncoding.has_key(k)):
			continue
		if coding[k]>0 and noncoding[k] >0:
			sum_of_log_ratio_2  +=  math.log( coding[k] / noncoding[k])
		elif coding[k]>0 and noncoding[k] == 0:
			sum_of_log_ratio_2 += 1
		elif coding[k] == 0 and noncoding[k] == 0:
			continue
		elif coding[k] == 0 and noncoding[k] >0 :
			sum_of_log_ratio_2 -= 1
		else:
			continue
		frame2_count += 1
	return max(sum_of_log_ratio_0/frame0_count, sum_of_log_ratio_1/frame1_count,sum_of_log_ratio_2/frame2_count)	
	'''
	return sum_of_log_ratio_0/frame0_count
	




#!/usr/bin/env python
'''
Equally divide BAM file (m alignments) into n parts. Each part contains roughly m/n alignments
that are randomly sampled from total alignments. 
'''

#import built-in modules
import os,sys
if sys.version_info[0] != 3:
    print("\nYou are using python" + str(sys.version_info[0]) + '.' + str(sys.version_info[1]) + " This verion of RSeQC needs python3!\n", file=sys.stderr)
    sys.exit()	

import string
from optparse import OptionParser
import warnings
import string
import collections
from random import randrange
#import third-party modules
import pysam

__author__ = "Liguo Wang"
__copyright__ = "Copyleft"
__credits__ = []
__license__ = "GPL"
__version__="3.0.0"
__maintainer__ = "Liguo Wang"
__email__ = "wang.liguo@mayo.edu"
__status__ = "Production"

def main():
	usage="%prog [options]" + '\n' + __doc__ + "\n"
	parser = OptionParser(usage,version="%prog " + __version__)
	parser.add_option("-i","--input-file",action="store",type="string",dest="input_file",help="Alignment file in BAM format. BAM file should be sorted and indexed.")
	parser.add_option("-n","--subset-num",action="store",type="int",dest="subset_num",help="Number of small BAM files")
	parser.add_option("-o","--out-prefix",action="store",type="string",dest="output_prefix",help="Prefix of output BAM files. Output \"Prefix_num.bam\".")
	parser.add_option("-s","--skip-unmap",action="store_true",dest="skip_unmap", help="Skip unmapped reads.")
	(options,args)=parser.parse_args()
		
	if not (options.input_file and options.subset_num and options.output_prefix):
		parser.print_help()
		sys.exit(0)
	if not os.path.exists(options.input_file):
		print('\n\n' + options.input_file + " does NOT exists" + '\n', file=sys.stderr)
		sys.exit(0)		
	
	samfile = pysam.Samfile(options.input_file,'rb')
	
	sub_bam = {}
	count_bam={}
	for i in range(0,options.subset_num):
		sub_bam[i] = pysam.Samfile(options.output_prefix + '_' + str(i) +'.bam','wb',template=samfile)
		count_bam[i] = 0
		
	total_alignment = 0
	print("Dividing " + options.input_file + " ...", end=' ', file=sys.stderr)
	try:
		while(1):
			aligned_read = next(samfile)
			if aligned_read.is_unmapped and options.skip_unmap is True:
				continue
			total_alignment += 1
			tmp = randrange(0,options.subset_num)
			sub_bam[tmp].write(aligned_read)
			count_bam[tmp] += 1
				
	except StopIteration:
		print("Done", file=sys.stderr)

	for i in range(0,options.subset_num):
		print("%-55s%d" %  (options.output_prefix + '_' + str(i) +'.bam', count_bam[i]))
				
if __name__ == '__main__':
	main()



#!/usr/bin/env python
'''
Annotate splicing reads against gene model in two levels: reads level and  juncion level.
Note:
1) A read, especially long read, can be spliced 2 or more times
2) Multiple splicing reads spanning the same intron can be consolidated into one splicing junction.
'''

#import built-in modules
import os,sys
if sys.version_info[0] != 3:
	print("\nYou are using python" + str(sys.version_info[0]) + '.' + str(sys.version_info[1]) + " This verion of RSeQC needs python3!\n", file=sys.stderr)
	sys.exit()	

import re
import string
from optparse import OptionParser
import warnings
import string
import collections
import math
from time import strftime
import subprocess

#import my own modules
from qcmodule import SAM
#changes to the paths

#changing history to this module


__author__ = "Liguo Wang"
__copyright__ = "Copyleft"
__credits__ = []
__license__ = "GPL"
__version__="3.0.0"
__maintainer__ = "Liguo Wang"
__email__ = "wang.liguo@mayo.edu"
__status__ = "Production"


def printlog (mesg):
        '''print progress into stderr and log file'''
        mesg="@ " + strftime("%Y-%m-%d %H:%M:%S") + ": " + mesg
        LOG=open('class.log','a')
        print(mesg, file=sys.stderr)
        print(mesg, file=LOG)

def generate_bed12(infile,size=10):
	''' 
	infile: input file. eg: chrX    66766604        66788677        348     partial_novel
	size: the block size representing exons
	'''
	
	outfile = infile.replace('.xls','.bed')
	OUT = open(outfile,'w')
	for line in open(infile,'r'):
		if line.startswith('chrom'):continue
		line = line.strip()
		f = line.split()
		if len(f) != 5:continue
		chrom = f[0]
		start = int(f[1]) - size
		end = int(f[2]) + size
		score = int(f[3])
		strand = '.'
		name = f[4]
		thick_st = start
		thick_end = end
		if name == 'annotated':
			color = '205,0,0'
		elif name == 'partial_novel':
			color = '0,205,0'
		elif name == 'complete_novel':
			color = '0,0,205'
		else:
			color = '0,0,0'
		blockCount = 2
		blockSizes = ','.join((str(size),str(size)))
		blockStarts = '0,' + str(end-size-start)
		print('\t'.join( [str(i) for i in [chrom, start, end, name, score, strand, thick_st, thick_end,color, blockCount, blockSizes,blockStarts]]), file=OUT)
		
	
def main():
	usage="%prog [options]" + '\n' + __doc__ + "\n"
	parser = OptionParser(usage,version="%prog " + __version__)
	parser.add_option("-i","--input-file",action="store",type="string",dest="input_file",help="Alignment file in BAM or SAM format.")
	parser.add_option("-r","--refgene",action="store",type="string",dest="ref_gene_model",help="Reference gene model in bed format. This file is better to be a pooled gene model as it will be used to annotate splicing junctions [required]")
	parser.add_option("-o","--out-prefix",action="store",type="string",dest="output_prefix",help="Prefix of output files(s). [required]")
	parser.add_option("-m","--min-intron",action="store",type="int",dest="min_intron",default=50, help="Minimum intron length (bp). default=%default [optional]")
	parser.add_option("-q","--mapq",action="store",type="int",dest="map_qual",default=30,help="Minimum mapping quality (phred scaled) for an alignment to be considered as \"uniquely mapped\". default=%default")

	(options,args)=parser.parse_args()
		
	if not (options.output_prefix and options.input_file and options.ref_gene_model):
		parser.print_help()
		sys.exit(0)
	if not os.path.exists(options.ref_gene_model):
		print('\n\n' + options.ref_gene_model + " does NOT exists" + '\n', file=sys.stderr)
		sys.exit(0)
	if os.path.exists(options.input_file):
		obj = SAM.ParseBAM(options.input_file)
		obj.annotate_junction(outfile=options.output_prefix,refgene=options.ref_gene_model,min_intron=options.min_intron, q_cut = options.map_qual)
		try:
			subprocess.call("Rscript " + options.output_prefix + '.junction_plot.r', shell=True)
		except:
			print("Cannot generate pdf file from " + '.junction_plot.r', file=sys.stderr)
			pass
	else:
		print('\n\n' + options.input_file + " does NOT exists" + '\n', file=sys.stderr)
		sys.exit(0)
	try:
		generate_bed12(options.output_prefix + '.junction.xls')	
	except:
		pass	


if __name__ == '__main__':
        main()



#!/usr/bin/env python
'''-------------------------------------------------------------------------------------------------
For each gene, check whether the RPKM value was saturated or not. Saturation analysis is based on 
re-sampling. For example, sample 5%, 10%, ... , 95%, 100% from total mapped reads, then 
calculate RPKM value for each step. Strand specific sequencing protocol is supported.
-------------------------------------------------------------------------------------------------'''

#import built-in modules
import os,sys
if sys.version_info[0] != 3:
	print("\nYou are using python" + str(sys.version_info[0]) + '.' + str(sys.version_info[1]) + " This verion of RSeQC needs python3!\n", file=sys.stderr)
	sys.exit()	


import re
import string
from optparse import OptionParser
import warnings
import collections
import math
import numpy as np
from time import strftime
import operator
import subprocess

#import third-party modules
from bx.bitset import *
from bx.bitset_builders import *
from bx.intervals import *

#import my own modules
from qcmodule import SAM
#changes to the paths

#changing history to this module


__author__ = "Liguo Wang"
__copyright__ = "Copyleft"
__credits__ = []
__license__ = "GPL"
__version__="3.0.0"
__maintainer__ = "Liguo Wang"
__email__ = "wang.liguo@mayo.edu"
__status__ = "Production"


def printlog (mesg):
	'''print progress into stderr and log file'''
	mesg="@ " + strftime("%Y-%m-%d %H:%M:%S") + ": " + mesg
	LOG=open('class.log','a')
	print(mesg, file=sys.stderr)
	print(mesg, file=LOG)

def normalize(lst):
	'''normalize all numbers between 0 and 1'''
	norm_lst=[]
	if max(lst) == min(lst):
		return norm_lst
	if max(lst) - min(lst)==0:
		return norm_lst
	for i in lst:
		norm_lst.append(  (i-min(lst))/(max(lst)-min(lst))  )  
	return norm_lst

def square_error(lst):
	'''transform list into normalized squared error (squared error divided by range)'''
	SE = []
	true_rpkm = lst[-1]
	rang = max(lst) - min(lst)
	if true_rpkm == 0:
		return None
	if rang == 0:
		return None
	for i in lst:
		SE.append(  abs(i - true_rpkm)/true_rpkm  )  
	return SE


def show_saturation (infile,outfile,rpkm_cut=0.01):
	
	RPKM_values = collections.defaultdict(list)
	RPKM_mean = {}
	gene_count = 0
	Quan = {'Q1':[0,0.25],'Q2':[0.25,0.5],'Q3':[0.5,0.75],'Q4':[0.75,1]}
	ROUT = open(outfile,'w')
	
	for line in open(infile):
		line=line.strip()
		fields=line.split()
		if fields[0].startswith('#'):
			head = [i.replace('%','') for i in fields[6:]]
			continue
		mykey = '\t'.join(fields[0:6])
		myvalue = [float(i) for i in fields[6:]]
		if max(myvalue) == 0: continue
		if max(myvalue) - min(myvalue) == 0: continue
		if np.mean(myvalue) < rpkm_cut: continue
		
		RPKM_values[mykey] = square_error(myvalue)
		RPKM_mean[mykey] = np.mean(myvalue)
		gene_count += 1
		if (len(head)==0):
			print("No head line found, exit.", file=sys.stderr)
			sys.exit(1)
	
	
	print("pdf('%s')" % (outfile.replace('.r','.pdf')), file=ROUT)
	print('par(mfrow=c(2,2))', file=ROUT)
	for quantile in sorted(Quan):
		line_count = 0
		norm_RPKM = collections.defaultdict(list)
		for k,v in sorted (iter(RPKM_mean.items()), key=operator.itemgetter(1)):	
			line_count += 1
			if (line_count > gene_count * Quan[quantile][0]) and (line_count <= gene_count * Quan[quantile][1]):
				for i,j in enumerate(RPKM_values[k]):
					norm_RPKM[head[i]].append(str(j))
		print("name=c(%s)" % (','.join(head[:-1])), file=ROUT)
		for i in head[:-1]:
			print("S%s=c(%s)" % (i, ','.join(norm_RPKM[i])), file=ROUT)
		print("boxplot(%s,names=name,outline=F,ylab='Percent Relative Error',main='%s',xlab='Resampling percentage')" % (','.join(['100*S' + i for i in head[:-1]]),quantile), file=ROUT)
	print('dev.off()', file=ROUT)
				

def main():
	usage="%prog [options]" + '\n' + __doc__ + "\n"
	parser = OptionParser(usage,version="%prog " + __version__)
	parser.add_option("-i","--input-file",action="store",type="string",dest="input_file",help="Alignment file in BAM or SAM format. [required]")
	parser.add_option("-o","--out-prefix",action="store",type="string",dest="output_prefix",help="Prefix of output files(s). [required]")
	parser.add_option("-r","--refgene",action="store",type="string",dest="refgene_bed",help="Reference gene model in bed fomat. [required]")
	parser.add_option("-d","--strand",action="store",type="string",dest="strand_rule",default=None,help="How read(s) were stranded during sequencing. For example: --strand='1++,1--,2+-,2-+' means that this is a pair-end, strand-specific RNA-seq, and the strand rule is: read1 mapped to '+' => parental gene on '+'; read1 mapped to '-' => parental gene on '-'; read2 mapped to '+' => parental gene on '-'; read2 mapped to '-' => parental gene on '+'.  If you are not sure about the strand rule, run \'infer_experiment.py' default=%default (Not a strand specific RNA-seq data)")
	parser.add_option("-l","--percentile-floor",action="store",type="int",dest="percentile_low_bound",default=5, help="Sampling starts from this percentile. A integer between 0 and 100. default=%default")
	parser.add_option("-u","--percentile-ceiling",action="store",type="int",dest="percentile_up_bound",default=100, help="Sampling ends at this percentile. A integer between 0 and 100. default=%default")
	parser.add_option("-s","--percentile-step",action="store",type="int",dest="percentile_step",default=5, help="Sampling frequency. Smaller value means more sampling times. A integer between 0 and 100. default=%default")	
	parser.add_option("-c","--rpkm-cutoff",action="store",type="float",dest="rpkm_cutoff",default=0.01, help="Transcripts with RPKM smaller than this number will be ignored in visualization plot. default=%default")
	parser.add_option("-q","--mapq",action="store",type="int",dest="map_qual",default=30,help="Minimum mapping quality (phred scaled) for an alignment to be called \"uniquely mapped\". default=%default")
	
	(options,args)=parser.parse_args()

	if not (options.output_prefix and options.input_file):
		parser.print_help()
		sys.exit(0)
	if options.percentile_low_bound <0 or options.percentile_low_bound >100:
		print("percentile_low_bound must be larger than 0 and samller than 100", file=sys.stderr)
		sys.exit(0)
	if options.percentile_up_bound <0 or options.percentile_up_bound >100:
		print("percentile_up_bound must be larger than 0 and samller than 100", file=sys.stderr)
		sys.exit(0)
	if options.percentile_up_bound < options.percentile_low_bound:
		print("percentile_up_bound must be larger than percentile_low_bound", file=sys.stderr)
		sys.exit(0)
	if options.percentile_step <0 or options.percentile_step > options.percentile_up_bound:
		print("percentile_step must be larger than 0 and samller than percentile_up_bound", file=sys.stderr)
		sys.exit(0)
	if os.path.exists(options.input_file):
		obj = SAM.ParseBAM(options.input_file)
		obj.saturation_RPKM(outfile=options.output_prefix, refbed=options.refgene_bed, sample_start=options.percentile_low_bound,sample_end=options.percentile_up_bound,sample_step=options.percentile_step,strand_rule=options.strand_rule, q_cut  = options.map_qual)
		show_saturation(infile=options.output_prefix + ".eRPKM.xls", outfile=options.output_prefix + ".saturation.r",rpkm_cut = options.rpkm_cutoff)
		try:
			subprocess.call("Rscript " + options.output_prefix + ".saturation.r", shell=True)
		except:
			pass
	else:
		print('\n\n' + options.input_file + " does NOT exists" + '\n', file=sys.stderr)
		#parser.print_help()
		sys.exit(0)
		


if __name__ == '__main__':
	main()



#!/usr/bin/env python
'''Manipulate two bigwig files'''
import os,sys

if sys.version_info[0] != 3:
	print("\nYou are using python" + str(sys.version_info[0]) + '.' + str(sys.version_info[1]) + " This verion of RSeQC needs python3!\n", file=sys.stderr)
	sys.exit()	

from optparse import OptionParser

import pyBigWig
from qcmodule import BED
from qcmodule import twoList
import numpy
__author__ = "Liguo Wang"
__copyright__ = "Copyleft"
__credits__ = []
__license__ = "GPL"
__version__="3.0.0"
__maintainer__ = "Liguo Wang"
__email__ = "wang.liguo@mayo.edu"
__status__ = "Testing"


def main():
	usage="%prog [options]"
	parser = OptionParser(usage,version="%prog " + __version__)
	
	parser.add_option("-i","--bwfile1",action="store",type="string",dest="BigWig_File1",help="One BigWig file.")
	parser.add_option("-j","--bwfile2",action="store",type="string",dest="BigWig_File2",help="Another BigWig file. Both BigWig files should use the same reference genome.")
	parser.add_option("-a","--action",action="store",type="string",dest="action",help='After pairwise align two bigwig files, perform the follow actions (Only select one keyword):"Add" = add signals. "Average" = average signals. "Division"= divide bigwig2 from bigwig1. Add 1 to both bigwig. "Max" = pick the signal that is larger. "Min" = pick the signal that is smaller. "Product" = multiply signals. "Subtract" = subtract signals in 2nd bigwig file from the corresponiding ones in the 1st bigwig file. "geometricMean" = take the geometric mean of signals.')
	parser.add_option("-o","--output",action="store",type="string",dest="output_wig",help="Output wig file")
	parser.add_option("-c","--chunk",action="store",type="int",dest="chunk_size",default=100000,help="Chromosome chunk size. Each chomosome will be cut into samll chunks of this size. Decrease chunk size will save more RAM. default=%default (bp)")
	(options,args)=parser.parse_args()
	
	if not (options.BigWig_File1 and options.BigWig_File2  and options.output_wig):
		parser.print_help()
		sys.exit(0)
	OUT=open(options.output_wig,'w')
	bw1   = pyBigWig.open(options.BigWig_File1)
	bw2   = pyBigWig.open(options.BigWig_File2)
	
	print("Get chromosome sizes from BigWig header ...", file=sys.stderr)
	chrom_sizes = {}
	for chr,size in bw1.chroms().items():
		chrom_sizes[chr] = size
	for chr,size in bw2.chroms().items():
		chrom_sizes[chr] = size
		
	for chr_name, chr_size in list(chrom_sizes.items()):		#iterate each chrom
		print("Processing " + chr_name + " ...", file=sys.stderr)
		OUT.write('variableStep chrom='+chr_name+'\n')
		for interval in BED.tillingBed(chrName = chr_name,chrSize = chr_size,stepSize = options.chunk_size):
			if (bw1.stats(chr_name,interval[1],interval[2] )[0] is None) and (bw2.stats(chr_name,interval[1],interval[2] )[0] is None):
				continue
			coord = interval[1]
			try:
				bw_signal1 = bw1.values(chr_name,interval[1],interval[2])
			except:
				bw_signal1 = numpy.array()
			try:
				bw_signal2 = bw2.values(chr_name,interval[1],interval[2])
			except:
				bw_signal2 = numpy.array()
			if bw_signal1 is None and bw_signal2 is None:
				continue
			if numpy.isnan(numpy.nansum(bw_signal1)) and numpy.isnan(numpy.nansum(bw_signal2)):
				continue
			if len(bw_signal1) == 0 and len(bw_signal2) == 0:
				continue
			bw_signal1 = numpy.nan_to_num( bw_signal1 )
			bw_signal2 = numpy.nan_to_num( bw_signal2 )
		
			call_back = getattr(twoList,options.action)
			for v in call_back(bw_signal1,bw_signal2):
				coord +=1
				if v != 0 : print("%d\t%.2f" % (coord,v), file=OUT)

if __name__=='__main__':
	main()



#!/usr/bin/env python
'''---------------------------------------------------------------------------------------
Calculate Hexamer frequency
------------------------------------------------------------------------------------------'''

import os,sys
import string
from optparse import OptionParser
import warnings
from qcmodule import FrameKmer

__author__ = "Liguo Wang"
__copyright__ = "Copyleft"
__credits__ = []
__license__ = "GPL"
__version__="3.0.0"
__maintainer__ = "Liguo Wang"
__email__ = "wang.liguo@mayo.edu"
__status__ = "Production"


def file_exist(file):
	try:
   		with open(file) as f: return True
	except IOError as e:
  	 return False


def main():
	usage = "\n%prog  [options]"
	parser = OptionParser(usage,version="%prog " + __version__)
	parser.add_option("-i","--input",action="store",dest="input_read",help="Read sequence in fasta or fastq format. Multiple fasta/fastq files should be separated by ','. For example: read.fq,read2.fa,read3,fa ")
	parser.add_option("-r","--refgenome",action="store",type="string",dest="ref_genome",help="Reference genome sequence in fasta format. Optional")
	parser.add_option("-g","--refgene",action="store",type="string",dest="ref_gene",help="Reference mRNA sequence in fasta format. Optional")
	(options,args)=parser.parse_args()
	
	if not options.input_read:
		parser.print_help()
		sys.exit(0)

	read_table={}
	read_file_names=[]	#base name
	read_file_sum = {}	#sum of hexamer
	
	for read_file in options.input_read.split(','):
		if not file_exist(read_file):
			print(read_file, ' does NOT exist!', file=sys.stderr)
			continue	
		print("Calculate hexamer of " + read_file + ' file ...', end=' ', file=sys.stderr)
		read_table[os.path.basename(read_file)] =  FrameKmer.kmer_freq_file(fastafile = read_file, word_size = 6, step_size = 1, frame = 0)
		read_file_names.append(os.path.basename(read_file))
		read_file_sum[os.path.basename(read_file)] = float(sum(read_table[os.path.basename(read_file)].values()))
		print("Done", file=sys.stderr)	
		
	if options.ref_genome and file_exist(options.ref_genome):
		print("Calculate hexamer of " + options.ref_genome + ' file ...', end=' ', file=sys.stderr)
		read_table[os.path.basename(options.ref_genome)] = FrameKmer.kmer_freq_file(fastafile = options.ref_genome, word_size = 6, step_size = 1, frame = 0)
		read_file_names.append(os.path.basename(options.ref_genome))
		read_file_sum[os.path.basename(options.ref_genome)] = float(sum(read_table[os.path.basename(options.ref_genome)].values()))
		print("Done.", file=sys.stderr)
		
	if options.ref_gene and file_exist(options.ref_gene):
		print("Calculate hexamer of " + options.ref_gene + ' file ...', end=' ', file=sys.stderr)
		read_table[os.path.basename(options.ref_gene)]= FrameKmer.kmer_freq_file(fastafile = options.ref_gene, word_size = 6, step_size = 1, frame = 0)	
		read_file_names.append(os.path.basename(options.ref_gene))
		read_file_sum[os.path.basename(options.ref_gene)] = float(sum(read_table[os.path.basename(options.ref_gene)].values()))
		print("Done.", file=sys.stderr)
	
	print('\n\nHexamer' + '\t' + '\t'.join(read_file_names))
		
	for kmer in FrameKmer.all_possible_kmer(6):
		if 'N' in kmer:continue
		print(kmer + '\t', end=' ')
		try:
			print('\t'.join([str(read_table[name][kmer] / (read_file_sum[name])) for name in read_file_names]))
		except:
			print('\t'.join([str(read_table[name][kmer] / (read_file_sum[name]+1)) for name in read_file_names]))
		
if __name__ == '__main__':
	main()



#!/usr/bin/env python
'''-------------------------------------------------------------------------------------------------
This program calculates transcript integrity number (TIN) for each transcript (or gene) in
BED file. TIN is conceptually similar to RIN (RNA integrity number) but provides transcript
level measurement of RNA quality and is more sensitive to measure low quality RNA samples:

1) TIN score of a transcript is used to measure the RNA integrity of the transcript.
2) Median TIN score across all transcripts can be used to measure RNA integrity  of that
   "RNA sample".
3) TIN ranges from 0 (the worst) to 100 (the best). TIN = 60 means: 60% of the transcript
   has been covered if the reads coverage were uniform.
4) TIN will be assigned to 0 if the transcript has no coverage or covered reads is fewer than
   cutoff.
-------------------------------------------------------------------------------------------------'''
import sys,os
import math,random
from optparse import OptionParser
from qcmodule import SAM
from qcmodule import getBamFiles
from numpy import mean,median,std
from time import strftime
import pysam
from bx.intervals import *

__author__ = "Liguo Wang"
__copyright__ = "Copyleft"
__credits__ = []
__license__ = "GPL"
__version__="3.0.0"
__maintainer__ = "Liguo Wang"
__email__ = "wang.liguo@mayo.edu"
__status__ = "Production"


def printlog (mesg):
	'''
	print mesg into stderr with time string appending to it.
	'''
	mesg="@ " + strftime("%Y-%m-%d %H:%M:%S") + ": " + mesg
	print(mesg, file=sys.stderr)

def uniqify(seq):
	'''
	duplicated members only keep one copy. [1,2,2,3,3,4] => [1,2,3,4].
	'''
	seen = set()
	return [x for x in seq if x not in seen and not seen.add(x)]

def shannon_entropy(arg):
	'''
	calculate shannon's H = -sum(P*log(P)). arg is a list of float numbers. Note we used
	natural log here.
	'''
	lst_sum = sum(arg)
	entropy = 0.0
	for i in arg:
		entropy += (i/lst_sum) * math.log(i/lst_sum)
	if entropy == 0:
		return 0
	else:
		return -entropy

def build_bitsets(list):
	'''
	build intevalTree from list
	'''
	ranges={}
	for l in list:
		chrom =l[0]
		st = l[1]
		end = l[2]
		if chrom not in ranges:
			ranges[chrom] = Intersecter()
		ranges[chrom].add_interval( Interval( st, end ) )
	return ranges

def union_exons(refbed):
	'''
	take the union of all exons defined in refbed file and build bitset
	'''
	from qcmodule import BED
	tmp = BED.ParseBED(refbed)
	all_exons = tmp.getExon()
	unioned_exons = BED.unionBed3(all_exons)
	exon_ranges = build_bitsets(unioned_exons)
	return exon_ranges

def estimate_bg_noise(chrom, tx_st, tx_end, samfile, e_ranges):
	'''
	estimate background noise level for a particular transcript
	'''
	intron_sig = 0.0	# reads_num * reads_len
	alignedReads = samfile.fetch(chrom,tx_st,tx_end)
	for aligned_read in alignedReads:
		if aligned_read.is_qcfail:continue 
		if aligned_read.is_unmapped:continue
		if aligned_read.is_secondary:continue
		read_start = aligned_read.pos
		if read_start < tx_st: continue
		if read_start >= tx_end: continue
		read_len = aligned_read.qlen
		if len(e_ranges[chrom].find(read_start, read_start + read_len)) > 0:
			continue
		intron_sig += read_len
	return intron_sig

def genomic_positions(refbed, sample_size):
	''' 
	return genomic positions of each nucleotide in mRNA. sample_size: Number of nucleotide
	positions sampled from mRNA.
	'''
	if refbed is None:
		print("You must specify a bed file representing gene model\n", file=sys.stderr)
		exit(0)
	
	for line in open(refbed,'r'):
		try:
			if line.startswith(('#','track','browser')):continue  
			# Parse fields from gene tabls
			fields = line.split()
			chrom     = fields[0]
			tx_start  = int( fields[1] )
			tx_end    = int( fields[2] )
			geneName      = fields[3]
			strand    = fields[5]
			cdsStart = int(fields[6]) + 1	#convert to 1-based
			cdsEnd = int(fields[7])
			exon_count = int(fields[9])
			mRNA_size = sum([int(i) for i in fields[10].strip(',').split(',')])
			geneID = '_'.join([str(j) for j in (chrom, tx_start, tx_end, geneName, strand)])
				
			exon_starts = list(map( int, fields[11].rstrip( ',\n' ).split( ',' ) ))
			exon_starts = list(map((lambda x: x + tx_start ), exon_starts))
			exon_ends = list(map( int, fields[10].rstrip( ',\n' ).split( ',' ) ))
			exon_ends = list(map((lambda x, y: x + y ), exon_starts, exon_ends))
			intron_size = tx_end - tx_start - mRNA_size
			if intron_size < 0:
				intron_size = 0
		except:
			print("[NOTE:input bed must be 12-column] skipped this line: " + line, end=' ', file=sys.stderr)
			continue
		
		chose_bases=[tx_start+1, tx_end]
		exon_bounds = []
		gene_all_base = []
		if mRNA_size <= sample_size:	# return all bases of mRNA
			for st,end in zip(exon_starts,exon_ends):
				chose_bases.extend(list(range(st+1,end+1)))		#1-based coordinates on genome, include exon boundaries
			yield (geneName, chrom, tx_start, tx_end, intron_size, chose_bases)
		elif mRNA_size > sample_size:
			step_size = int(mRNA_size/sample_size)
			for st,end in zip(exon_starts,exon_ends):
				gene_all_base.extend(list(range(st+1,end+1)))
				exon_bounds.append(st+1)
				exon_bounds.append(end)
			indx = list(range(0, len(gene_all_base), step_size))
			chose_bases = [gene_all_base[i] for i in indx]
			yield (geneName, chrom, tx_start, tx_end, intron_size, uniqify(exon_bounds + chose_bases))
		
def check_min_reads(samfile,chrom,tx_st, tx_end, cutoff):
	'''
	make sure the gene has minimum reads coverage. if cutoff = 10, each gene must have
	10 *different* reads.
	'''
	tmp = False
	read_count = set()
	try:
		alignedReads = samfile.fetch(chrom,tx_st,tx_end)
		for aligned_read in alignedReads:
			if aligned_read.is_qcfail:continue 
			if aligned_read.is_unmapped:continue
			if aligned_read.is_secondary:continue
			read_start = aligned_read.pos
			if read_start < tx_st: continue
			if read_start >= tx_end: continue
			read_count.add(read_start)
			if len(read_count) > cutoff:	#no need to loop anymore
				tmp = True
				break
		return tmp
	except:
		return False

	
def genebody_coverage(samfile, chrom, positions, bg_level = 0):
	'''
	calculate coverage for each nucleotide in *positions*. some times len(cvg) < len(positions)
	because positions where there is no mapped reads were ignored.
	'''
	cvg = []
	start = positions[0] - 1
	end = positions[-1]
	
	try:
		for pileupcolumn in samfile.pileup(chrom, start, end, truncate=True):
			ref_pos = pileupcolumn.pos+1
			if ref_pos not in positions: continue
			if pileupcolumn.n == 0:
				cvg.append(0.0)
				continue
			cover_read = 0.0
			for pileupread in pileupcolumn.pileups:
				if pileupread.is_del: continue
				if pileupread.alignment.is_qcfail:continue 
				if pileupread.alignment.is_secondary:continue 
				if pileupread.alignment.is_unmapped:continue
				#if pileupread.alignment.is_duplicate:continue
				cover_read +=1.0
			cvg.append(cover_read)
	except:
		cvg = []
	
	if bg_level <= 0:
		return cvg
	else:
		tmp = []
		for i in cvg:
			subtracted_sig = int(i - bg_level)
			if subtracted_sig > 0:
				tmp.append(subtracted_sig)
			else:
				tmp.append(0)
		return tmp


def tin_score(cvg, l):
	'''calcualte TIN score'''
	
	if len(cvg) == 0:
		tin = 0
		return tin
	
	cvg_eff = [float(i) for i in cvg if float(i) > 0]	#remove positions with 0 read coverage
	entropy = shannon_entropy(cvg_eff)
	
	tin = 100*(math.exp(entropy)) / l
	return tin


def main():
	usage="%prog [options]" + '\n' + __doc__ + "\n"
	parser = OptionParser(usage,version="%prog " + __version__)
	parser.add_option("-i","--input",action="store",type="string",dest="input_files",help='Input BAM file(s). "-i" takes these input: 1) a single BAM file. 2) "," separated BAM files (no spaces allowed). 3) directory containing one or more bam files. 4) plain text file containing the path of one or more bam files (Each row is a BAM file path). All BAM files should be sorted and indexed using samtools. [required]')
	parser.add_option("-r","--refgene",action="store",type="string",dest="ref_gene_model",help="Reference gene model in BED format. Must be strandard 12-column BED file. [required]")
	parser.add_option("-c","--minCov",action="store",type="int",dest="minimum_coverage",default=10,help="Minimum number of read mapped to a transcript. default=%default")
	parser.add_option("-n","--sample-size",action="store",type="int",dest="sample_size",default=100,help="Number of equal-spaced nucleotide positions picked from mRNA. Note: if this number is larger than the length of mRNA (L), it will be halved until it's smaller than L. default=%default")
	parser.add_option("-s","--subtract-background",action="store_true",dest="subtract_bg",help="Subtract background noise (estimated from intronic reads). Only use this option if there are substantial intronic reads.")
	(options,args)=parser.parse_args()
	
	# if '-s' was set
	if options.subtract_bg:
		exon_ranges = union_exons(options.ref_gene_model)
		
	if options.sample_size < 0:
		print("Number of nucleotide can't be negative", file=sys.stderr)
		sys.exit(0)
	elif options.sample_size >1000:
		print("Warning: '-n' is too large! Please try smaller '-n' valeu if program is running slow.", file=sys.stderr)
		
	if not (options.input_files and options.ref_gene_model):
		parser.print_help()
		sys.exit(0)

	if not os.path.exists(options.ref_gene_model):
		print('\n\n' + options.ref_gene_model + " does NOT exists" + '\n', file=sys.stderr)
		parser.print_help()
		sys.exit(0)
		
	printlog("Get BAM file(s) ...")
	bamfiles = sorted(getBamFiles.get_bam_files(options.input_files))
	
	if len(bamfiles) <= 0:
		print("No BAM file found, exit.", file=sys.stderr)
		sys.exit(0)
	else:
		print("Total %d BAM file(s):" % len(bamfiles), file=sys.stderr)
		for f in bamfiles:
			print("\t" + f, file=sys.stderr)	
	
	
	for f in bamfiles:
		printlog("Processing " + f)
		
		SUM = open(os.path.basename(f).replace('bam','') + 'summary.txt','w')
		print("\t".join(['Bam_file','TIN(mean)', 'TIN(median)','TIN(stdev)']), file=SUM)
		
		OUT = open(os.path.basename(f).replace('bam','') + 'tin.xls','w')
		print("\t".join(["geneID","chrom", "tx_start", "tx_end","TIN"]), file=OUT)
		
		samfile = pysam.Samfile(f, "rb")
		sample_TINs = []	#sample level TIN, values are from different genes
		finish = 0
		noise_level = 0.0
		for gname, i_chr, i_tx_start, i_tx_end, intron_size, pick_positions in genomic_positions(refbed = options.ref_gene_model, sample_size = options.sample_size):	
			finish += 1
			
			# check minimum reads coverage
			if check_min_reads(samfile,i_chr,i_tx_start,i_tx_end,options.minimum_coverage) is not True:
				print('\t'.join([str(i) for i in (gname, i_chr, i_tx_start, i_tx_end, 0.0)]), file=OUT)
				continue
				
			# estimate background noise if '-s' was specified
			if options.subtract_bg:
				intron_signals = estimate_bg_noise(i_chr, i_tx_start, i_tx_end, samfile, exon_ranges)
				if intron_size > 0:
					noise_level = intron_signals/intron_size					

			coverage = genebody_coverage(samfile, i_chr,sorted(pick_positions), noise_level)
			
			#for a,b in zip(sorted(pick_positions),coverage):
			#	print str(a) + '\t' + str(b)
			
			tin1 = tin_score(cvg = coverage, l = len(pick_positions))
			sample_TINs.append(tin1)
			print('\t'.join([str(i) for i in (gname, i_chr, i_tx_start, i_tx_end, tin1)]), file=OUT)
			print(" %d transcripts finished\r" % (finish), end=' ', file=sys.stderr)
		
		print("\t".join( [str(i) for i in (os.path.basename(f), mean(sample_TINs), median(sample_TINs), std(sample_TINs))]), file=SUM)
		OUT.close()
		SUM.close()
		samfile.close()

if __name__ == '__main__':
	
	main()

	



"""
File    quantile.py
Desc    computes sample quantiles
Author  Ernesto P. Adorio, PhD.
        UPDEPP (U.P. at Clarkfield)
Version 0.0.1 August 7. 2009
"""

from math import modf, floor
def quantile(x, q,  qtype = 7, issorted = False):
    """
    Args:
       x - input data
       q - quantile
       qtype - algorithm
       issorted- True if x already sorted.
    Compute quantiles from input array x given q.For median,
    specify q=0.5.
    References:
       http://reference.wolfram.com/mathematica/ref/Quantile.html
       http://wiki.r-project.org/rwiki/doku.php?id=rdoc:stats:quantile
    Author:
    Ernesto P.Adorio Ph.D.
    UP Extension Program in Pampanga, Clark Field.
    """
    if not issorted:
        y = sorted(x)
    else:
        y = x
    if not (1 <= qtype <= 9):
       return None  # error!
    # Parameters for the Hyndman and Fan algorithm
    abcd = [(0,   0, 1, 0), # inverse empirical distrib.function., R type 1
            (0.5, 0, 1, 0), # similar to type 1, averaged, R type 2
            (0.5, 0, 0, 0), # nearest order statistic,(SAS) R type 3
            (0,   0, 0, 1), # California linear interpolation, R type 4
            (0.5, 0, 0, 1), # hydrologists method, R type 5
            (0,   1, 0, 1), # mean-based estimate(Weibull method), (SPSS,Minitab), type 6
            (1,  -1, 0, 1), # mode-based method,(S, S-Plus), R type 7
            (1.0/3, 1.0/3, 0, 1), # median-unbiased ,  R type 8
            (3/8.0, 0.25, 0, 1)   # normal-unbiased, R type 9.
           ]
    a, b, c, d = abcd[qtype-1]
    n = len(x)
    g, j = modf( a + (n+b) * q -1)
    if j < 0:
        return y[0]
    elif j >= n:
        return y[n-1]   # oct. 8, 2010 y[n]???!! uncaught  off by 1 error!!!
    j = int(floor(j))
    if g ==  0:
       return y[j]
    else:
       return y[j] + (y[j+1]- y[j])* (c + d * g)

def Test():
    x = [11.4, 17.3, 21.3, 25.9, 40.1, 50.5, 60.0, 70.0, 75]
    for qtype in range(1,10):
        print(qtype, quantile(x, 0.35, qtype))


if __name__ == "__main__":
    Test()



"""
Get bam files from input, input could be:
1) directory that containing one or more bam files
2) plain text file containing one or more bam file paths
3) a single bam file
4) ',' separated multiple bam files

in all cases, the index .bai file(s) should be exist in the same location. eg, if test.bam
exists, test.bam.bai must also exist.

This python3 module was converted from python2.7 code using 2to3

"""

import os,sys
from os.path import abspath,join, getsize

__author__ = "Liguo Wang"
__copyright__ = "Copyleft"
__credits__ = []
__license__ = "GPL"
__version__="3.0.0"
__maintainer__ = "Liguo Wang"
__email__ = "wang.liguo@mayo.edu"
__status__ = "Production"


def isbamfile(infile):
    '''check if it is bam file, if it is empty and if the .bai file exists'''
    if os.path.isfile(infile) and infile[-4:].lower() == '.bam':
        if getsize(infile) != 0:
            if os.path.isfile(infile + '.bai'):
                return True
            else:
                print("Warning: %s.bai does not exist! Skip it." % (infile), file=sys.stderr)
                return False
        else:
            print("The size of %s is 0! Skip it." % (infile), file=sys.stderr)
            return False
    else:
        return False    

def get_bam_files (input,printit=False):
    bam_files = []  
    
    #dir
    if os.path.isdir(input):
        for root, directories, files in os.walk(input,followlinks=True):
            full_names = [join(abspath(root), name) for name in files]
            for fn in full_names:
                if isbamfile(fn):
                    bam_files.append(fn)
    #single bam file
    elif isbamfile(input):
        bam_files.append(input)
    #plain text file
    elif os.path.isfile(input):
        try:
            for line in open(input):
                line = line.strip()
                if line.startswith('#'):continue
                if isbamfile(line):
                    bam_files.append(line)
        except:
            pass
    else:
        tmp = input.split(',')
        if len(tmp) <2: pass
        for i in tmp:
            if isbamfile(i):
                bam_files.append(i)
    
    if printit:
        for i in bam_files:
            print(i)        
    return bam_files


if __name__ == '__main__':
    get_bam_files(sys.argv[1])


#!/usr/bin/env python
'''-------------------------------------------------------------------------------------------------
Split bam file (pair-end) into 2 single-end bam file
-------------------------------------------------------------------------------------------------'''

#import built-in modules
import os,sys
if sys.version_info[0] != 3:
	print("\nYou are using python" + str(sys.version_info[0]) + '.' + str(sys.version_info[1]) + " This verion of RSeQC needs python3!\n", file=sys.stderr)
	sys.exit()	


import string
from optparse import OptionParser
import warnings
import collections

#import third-party modules
from bx.bitset import *
from bx.bitset_builders import *
from bx.intervals import *
from bx.binned_array import BinnedArray
from bx_extras.fpconst import isNaN
from bx.bitset_utils import *
import pysam

#import my own modules
from qcmodule import BED
from qcmodule import SAM
from qcmodule import bam_cigar

__author__ = "Liguo Wang"
__copyright__ = "Copyleft"
__credits__ = []
__license__ = "GPL"
__version__="3.0.0"
__maintainer__ = "Liguo Wang"
__email__ = "wang.liguo@mayo.edu"
__status__ = "Production"

	
def main():
	usage="%prog [options]" + '\n' + __doc__ + "\n"
	parser = OptionParser(usage,version="%prog " + __version__)
	parser.add_option("-i","--input-file",action="store",type="string",dest="input_file",help="Alignment file in BAM or SAM format. BAM file should be sorted and indexed")
	parser.add_option("-o","--out-prefix",action="store",type="string",dest="output_prefix",help="Prefix of output BAM files. \"prefix.R1.bam\" file contains the 1st read, \"prefix.R2.bam\" file contains the 2nd read")
	(options,args)=parser.parse_args()
		
	if not (options.input_file):
		parser.print_help()
		sys.exit(0)
	if not os.path.exists(options.input_file):
		print('\n\n' + options.input_file + " does NOT exists" + '\n', file=sys.stderr)
		sys.exit(0)		
		
	samfile = pysam.Samfile(options.input_file,'rb')
	OUT1 = pysam.Samfile(options.output_prefix + '.R1.bam','wb',template=samfile)	#bam file containing reads hit to exon region
	OUT2 = pysam.Samfile(options.output_prefix + '.R2.bam','wb',template=samfile)	#bam file containing reads not hit to exon region
	OUT3 = pysam.Samfile(options.output_prefix + '.unmap.bam','wb',template=samfile)	#bam file containing reads not hit to exon region
	
	total_alignment = 0
	r1_alignment = 0
	r2_alignment = 0
	unmapped = 0
	
	print("spliting " + options.input_file + " ...", end=' ', file=sys.stderr)
	try:
		while(1):
			new_alignment = pysam.AlignedRead()     # create AlignedRead object
			old_alignment = next(samfile)
			total_alignment += 1
			
			new_alignment.qname = old_alignment.qname      # 1st column. read name.
			#new_alignment.flag = old_alignment.flag        # 2nd column. subject to change. flag value 
			new_alignment.tid = old_alignment.tid          # 3rd column. samfile.getrname(tid) == chrom name
			new_alignment.pos = old_alignment.pos          # 4th column. reference Start position of the aligned part (of read) [0-based]
			new_alignment.mapq = old_alignment.mapq        # 5th column. mapping quality
			new_alignment.cigar= old_alignment.cigar       # 6th column. subject to change. 
			#new_alignment.rnext = old_alignment.rnext      # 7th column. tid of the reference (mate read mapped to)
			#new_alignment.pnext = old_alignment.pnext      # 8th column. position of the reference (0 based, mate read mapped to)
			#new_alignment.tlen = old_alignment.tlen        # 9th column. insert size
			new_alignment.seq = old_alignment.seq          # 10th column. read sequence. all bases.
			new_alignment.qual = old_alignment.qual        # 11th column. read sequence quality. all bases.
			new_alignment.tags = old_alignment.tags        # 12 - columns
			new_alignment.flag = 0x0000
			if old_alignment.is_unmapped:
				OUT3.write(old_alignment)
				unmapped += 1
				continue
			if old_alignment.is_reverse:
				new_alignment.flag = new_alignment.flag | 0x0010

			if old_alignment.is_secondary:
				new_alignment.flag = new_alignment.flag | 0x0100
			if old_alignment.is_qcfail:
				new_alignment.flag = new_alignment.flag | 0x0200
			if old_alignment.is_duplicate:
				new_alignment.flag = new_alignment.flag | 0x0400
			if old_alignment.is_read1:
				OUT1.write(new_alignment)
				r1_alignment += 1
			else:
				OUT2.write(new_alignment)
				r2_alignment += 1

	except StopIteration:
		print("Done", file=sys.stderr)
				
	print("%-55s%d" % ("Total records:",total_alignment))
	print("%-55s%d" % (options.output_prefix + 'Read 1:',r1_alignment))
	print("%-55s%d" % (options.output_prefix + 'Read 2:',r2_alignment))
	print("%-55s%d" % (options.output_prefix + 'Unmapped:',unmapped))
	
if __name__ == '__main__':
	main()



#!/usr/bin/env python
'''
This python3 module was converted from python2.7 code using 2to3
'''

#import built-in modules
import re
import sys
from random import shuffle
from heapq import nlargest
#import third-party modules

#changes to the paths

#changing history to this module



__author__ = "Liguo Wang"
__copyright__ = "Copyleft"
__credits__ = []
__license__ = "GPL"
__version__="3.0.0"
__maintainer__ = "Liguo Wang"
__email__ = "wang.liguo@mayo.edu"
__status__ = "Production"


def S_diff(lst):
    '''Given a list of int or float, calculate S_diff and S_point'''
    
    S_avg = sum(lst) / len(lst)
    S_dist = [i-S_avg for i in lst] #distance to average
    S_cum=[]    #list of cumulative sum
    S_cum.append(0)
    for i in range(0,len(S_dist)):
        S_cum.append(S_cum[i] + S_dist[i])
    return [nlargest(1,list(range(0,len(S_cum))),key=lambda i: S_cum[i]),(max(S_cum) - min(S_cum))]
    #return the index of maximum_diff index, and maximum_diff

def bootstrap(lst,obs,rep=1000):
    '''Given a list of int or float (lst) and an observation value(obs). calcualte the chance (pvalue) 
    of getting this observation through bootstrapping.'''
    
    shuffled_diff=[]
    count=0
    tmp=lst
    for i in range(0,rep):
        shuffle(tmp)
        shuffled_diff.append(S_diff(tmp))
    
    for i in sorted(shuffled_diff):
        if (i>=obs):
            count += 1 
    if count/rep <0.5:
        return count/rep
    else:
        return 1- count/rep



#!/usr/bin/env python
#Liguo Wang
#04/13/2011

'''
This python3 module was converted from python2.7 code using 2to3
'''

#import built-in modules
import os,sys
import re
import string
#from optparse import OptionParser
import warnings
import string
import collections
import math

#import third-party modules
#from bx.bitset import *
#from bx.bitset_builders import *
#from bx.intervals import *
#import fasta
import bx.wiggle
from bx.binned_array import BinnedArray
from bx_extras.fpconst import isNaN

__author__ = "Liguo Wang"
__copyright__ = "Copyleft"
__credits__ = []
__license__ = "GPL"
__version__="3.0.0"
__maintainer__ = "Liguo Wang"
__email__ = "wang.liguo@mayo.edu"
__status__ = "Production"


class ParseWig:
    '''provie methods to manipulate wiggle format file. For wiggle format see:
    http://genome.ucsc.edu/goldenPath/help/wiggle.html'''
    
    def __init__(self,wigFile):
        '''read wig file, creat wig obj'''
        self.scores = {}
        self.num_re=re.compile(r'[\d\.\-\+]+');
        fh=open(wigFile)
        #infile=open(wigFile,'r')
        for i, ( chrom, pos, val ) in enumerate( bx.wiggle.Reader( fh ) ):
            chrom=chrom.upper()
            if not chrom in self.scores: self.scores[ chrom ] = BinnedArray()
            self.scores[chrom][pos] = val
            if i % 100000 == 0: print("%i datapoints loaded \r" % i) 
        #print self.scores.keys()
        print("total " + str(i) + " points loaded")
        
    def fetch_all_scores(self,chr,st,end):
        '''fetch all wiggle scores defined by st and end.  NOTE:
        1)st and end are 0-based, half-open. (st,end]
        2)points without score are indicated as "nan"
        '''
        chr=chr.upper()
        return [ self.scores[chr][i] for i in range(st,end)]

    def fetch_max_scores(self,chr,st,end):
        ''' fetch maximum score defined by chr, st, end
        1)st and end are 0-based, half-open. (st,end]
        '''     
        
        chr=chr.upper()
        return max([ self.scores[chr][i] for i in range(st,end)])

    def fetch_min_scores(self,chr,st,end):
        ''' fetch minimum score defined by chr, st, end
        1)st and end are 0-based, half-open. (st,end]
        ''' 
        
        chr=chr.upper()
        return min([ self.scores[chr][i] for i in range(st,end)])           
    
    def fetch_avg_scores(self,chr,st,end):
        ''' fetch average score defined by chr, st, end
        1)st and end are 0-based, half-open. (st,end]
        ''' 
        
        chr=chr.upper()
        lst=[ float(self.scores[chr][i]) for i in range(st,end) if self.num_re.match(str(self.scores[chr][i]))]
        return sum(lst)/len(list(range(st,end)))
        
    def fetch_sum_scores(self,chr,st,end):
        ''' fetch sum score defined by chr, st, end
        1)st and end are 0-based, half-open. (st,end]
        ''' 
        
        chr=chr.upper()
        lst=[ float(self.scores[chr][i]) for i in range(st,end) if self.num_re.match(str(self.scores[chr][i]))]
        return sum(lst)     
#if __name__ == "__main__": main()

class ParseWig2:
    '''provie methods to manipulate wiggle format file. For wiggle format see:
    http://genome.ucsc.edu/goldenPath/help/wiggle.html. The same coordinate could occur more than
    one time in wig file, and the scores will be sumed up. Slower than ParseWig'''
    
    def __init__(self,wigFile):
        '''read wig file, creat wig obj'''
        self.scores = {}
        self.num_re=re.compile(r'[\d\.\-\+]+');
        fh=open(wigFile)
        #infile=open(wigFile,'r')
        for i, ( chrom, pos, val ) in enumerate( bx.wiggle.Reader( fh ) ):
            chrom=chrom.upper()
            if not chrom in self.scores: self.scores[chrom] = BinnedArray()
            tmp=self.scores[chrom][pos]
            if isNaN(tmp):
                self.scores[chrom][pos] = val
            else:
                self.scores[chrom][pos] += val
            if i % 100000 == 0: print("%i datapoints loaded \r" % i) 
        #print self.scores.keys()
        print("total " + str(i) + " points loaded")
        
    def fetch_all_scores_by_range(self,chr,st,end):
        '''fetch all wiggle scores defined by st and end.  NOTE:
        1)st and end are 0-based, half-open. (st,end]
        2)points without score are indicated as "nan"'''
        chr=chr.upper()
        return [ self.scores[chr][i] for i in range(st,end)]
        
    def fetch_all_scores_by_positions(self,chr,lst):
        '''fetch all wiggle scores defined by st and end.  NOTE:
        2)points without score are indicated as "nan"'''
        chr=chr.upper()
        return [ self.scores[chr][i] for i in lst]
        
    def fetch_max_scores_by_range(self,chr,st,end):
        ''' fetch maximum score defined by chr, st, end
        1)st and end are 0-based, half-open. (st,end]
        '''         
        chr=chr.upper()
        return max([ self.scores[chr][i] for i in range(st,end)])

    def fetch_max_scores_by_positions(self,chr,lst):
        '''fetch maximum score defined by chr, st, end'''
        
        chr=chr.upper()
        return max([ self.scores[chr][i] for i in lst])

    def fetch_min_scores_by_range(self,chr,st,end):
        ''' fetch minimum score defined by chr, st, end
        1)st and end are 0-based, half-open. (st,end]
        '''         
        chr=chr.upper()
        return min([ self.scores[chr][i] for i in range(st,end)])           

    def fetch_min_scores_by_positions(self,chr,lst):
        ''' fetch minimum score defined by chr, st, end
        '''         
        chr=chr.upper()
        return min([ self.scores[chr][i] for i in lst])
        
    
    def fetch_avg_scores_by_range(self,chr,st,end):
        ''' fetch average score defined by chr, st, end
        1)st and end are 0-based, half-open. (st,end]
        '''         
        chr=chr.upper()
        lst=[ float(self.scores[chr][i]) for i in range(st,end) if self.num_re.match(str(self.scores[chr][i]))]
        return sum(lst)/len(list(range(st,end)))
        
    def fetch_avg_scores_by_positions(self,chr,lst):
        ''' fetch average score defined by chr, st, end
        '''         
        chr=chr.upper()
        lst_score =[ float(self.scores[chr][i]) for i in lst if self.num_re.match(str(self.scores[chr][i]))]
        return sum(lst_score)/len(lst_score)
        
    def fetch_sum_scores_by_range(self,chr,st,end):
        ''' fetch sum score defined by chr, st, end
        '''         
        chr=chr.upper()
        lst=[ float(self.scores[chr][i]) for i in range(st,end) if self.num_re.match(str(self.scores[chr][i]))]
        return sum(lst)     

    def fetch_sum_scores_by_positions(self,chr,lst):
        ''' fetch sum score defined by chr, st, end
        '''         
        chr=chr.upper()
        lst_score=[ float(self.scores[chr][i]) for i in lst if self.num_re.match(str(self.scores[chr][i]))]
        return sum(lst_score)

    def distriub_wig(self,bed,till_count=100):
        '''calculate coverage over bed file (only consider exon regions). The mRNA sequences in input
        bed file will be cut into 100 tills of equal size'''
        
        print("Reading " + bed + " ...", file=sys.stderr)
        for line in open(bed,'r'):
            try:
                if line.startswith(('#','track','browser')):continue
                fields=line.rstrip('\r\n').split()
                txStart=int(fields[1])
                chrom=fields[0]
                strand=fields[5]
                geneName=fields[3]
                score=fields[4]
                exon_start=list(map(int,fields[11].rstrip(',').split(',')))
                exon_start=list(map((lambda x: x + txStart),exon_start))
                exon_end=list(map(int,fields[10].rstrip(',').split(',')))
                exon_end=list(map((lambda x,y:x+y),exon_start,exon_end))
            except:
                print("[NOTE:input bed must be 12-column] skipped this line: " + line, end=' ', file=sys.stderr)
                continue
        
#if __name__ == "__main__": main()



'''
This python3 module was converted from python2.7 code using 2to3
'''

import collections
import sys

__author__ = "Liguo Wang"
__copyright__ = "Copyleft"
__credits__ = []
__license__ = "GPL"
__version__="3.0.0"
__maintainer__ = "Liguo Wang"
__email__ = "wang.liguo@mayo.edu"
__status__ = "Production"



start_coden = ['ATG']
stop_coden = ["TAG","TAA","TGA"]
def _reverse_comp(seq):
    swap = {"A":"T", "T":"A", "C":"G", "G":"C","N":"N","X":"X"}
    tmp = "".join(swap[b] for b in seq)
    return tmp[::-1]
    
def longest_orf(seq,strandness,sc=None,tc=None):
    '''find the longest ORF in input mRNA sequence. strand=0 only search '+' strand, strand=1, only 
    search '-' strand, strand=2, search both strand'''
    orf_ranges=collections.defaultdict(list)
    dna_seq = seq.upper()
    possible_orf={} #[orf-st, orf_end] ==>size
    largest_orf=""
    
    if sc is not None:
        start_coden = sc.strip(',').split(',')
        if len(start_coden)==0:
            print("Unkown start codon", file=sys.stderr)
            sys.exit(1)
        else:
            for cd in start_coden:
                if len(cd) != 3:
                    print("Unkown start codon" + str(cd), file=sys.stderr)
                    sys.exit(1)
    if tc is not None:
        stop_coden = tc.strip(',').split(',')
        if len(stop_coden)==0:
            print("Unkown stop codon", file=sys.stderr)
            sys.exit(1)
        else:
            for cd in stop_coden:
                if len(cd) != 3:
                    print("Unkown stop codon" + str(cd), file=sys.stderr)
                    sys.exit(1) 

    strand = strandness
    start_pos = []
    end_pos=[]
    orf_ranges.clear()
    if strand == '-':
        dna_seq=_reverse_comp(dna_seq)
    for sc in start_coden:
        start_found = dna_seq.find(sc)
        while start_found >-1:
            start_pos.append(start_found)
            start_found = dna_seq.find(sc,start_found +1)
    for sc in stop_coden:
        end_found = dna_seq.find(sc)
        while end_found >-1:
            end_pos.append(end_found)
            end_found = dna_seq.find(sc,end_found+1)
    
    for st in start_pos:
        for end in end_pos:
            if end <= st: continue
            if (end - st) % 3==0:
                orf_ranges[st].append(end)
    for k in sorted(orf_ranges):
        possible_orf[str(k) + '\t' + str(min(orf_ranges[k])) + '\t' + strand] = min(orf_ranges[k]) - k
        
        
    for k,v in list(possible_orf.items()):
        if v == max(possible_orf.values()):
            #print "#" + k
            fields=k.split()
            largest_orf = dna_seq[int(fields[0]):int(fields[1])]
    return largest_orf      #could be None, '' or DNA sequencee
    
                
def longest_orf_bed(seq,bedline,sc=None,tc=None):
    '''find the longest ORF in input mRNA sequence. strand=0 only search '+' strand, strand=1, only 
    search '-' strand, strand=2, search both strand'''
    orf_ranges=collections.defaultdict(list)
    dna_seq = seq.upper()
    possible_orf={} #[orf-st, orf_end] ==>size
    largest_orf=""

    if sc is not None:
        start_coden = sc.strip(',').split(',')
        if len(start_coden)==0:
            print("Unkown start codon", file=sys.stderr)
            sys.exit(1)
        else:
            for cd in start_coden:
                if len(cd) != 3:
                    print("Unkown start codon" + str(cd), file=sys.stderr)
                    sys.exit(1)
    if tc is not None:
        stop_coden = tc.strip(',').split(',')
        if len(stop_coden)==0:
            print("Unkown stop codon", file=sys.stderr)
            sys.exit(1)
        else:
            for cd in stop_coden:
                if len(cd) != 3:
                    print("Unkown stop codon" + str(cd), file=sys.stderr)
                    sys.exit(1) 
    
    fields = bedline.split()
    txStart = int(fields[1])
    exon_sizes = [int(i) for i in fields[10].rstrip(',\n').split(',')]
    exon_starts = list(map(int,fields[11].rstrip(',').split(',')))
    exon_starts = list(map((lambda x: x + txStart),exon_starts))
    exon_ends = list(map(int,fields[10].rstrip(',').split(',')))
    exon_ends = list(map((lambda x,y:x+y),exon_starts,exon_ends))
    strand = fields[5]
    
    start_pos = []
    end_pos=[]
    orf_ranges.clear()
    if strand == '-':
        dna_seq=_reverse_comp(dna_seq)
    for sc in start_coden:
        start_found = dna_seq.find(sc)
        while start_found >-1:
            start_pos.append(start_found)
            start_found = dna_seq.find(sc,start_found +1)
    for sc in stop_coden:
        end_found = dna_seq.find(sc)
        while end_found >-1:
            end_pos.append(end_found)
            end_found = dna_seq.find(sc,end_found+1)
    
    for st in start_pos:
        for end in end_pos:
            if end <= st: continue
            if (end - st) % 3==0:
                orf_ranges[st].append(end)
    for k in sorted(orf_ranges):
        possible_orf[str(k) + '\t' + str(min(orf_ranges[k])) + '\t' + strand] = min(orf_ranges[k]) - k
    
    for k,v in list(possible_orf.items()):
        if v == max(possible_orf.values()):
            #print k + '\t' + str(v)
            col=k.split()
            cds_st = int(col[0])
            cds_end = int(col[1])
            strand = col[2]
            
            if strand =='+':
                #determine CDS start site
                for exon_size, exon_st in zip(exon_sizes,exon_starts):
                    if cds_st > exon_size: 
                        cds_st = cds_st - exon_size
                        continue
                    else:
                        cds_pos1 = exon_st + cds_st
                        break
                #determine CDS end site
                for exon_size, exon_st in zip(exon_sizes,exon_starts):
                    if cds_end > exon_size:
                        cds_end = cds_end - exon_size
                        continue
                    else:
                        cds_pos2 = exon_st + cds_end + 3
                        break
                fields[6] = str(cds_pos1)
                fields[7] = str(cds_pos2)
                return '\t'.join(fields)

            if strand =='-':
                exon_sizes = exon_sizes[::-1]
                exon_ends = exon_ends[::-1]
                for exon_size, exon_end in zip(exon_sizes,exon_ends):
                    if cds_st > exon_size: 
                        cds_st = cds_st - exon_size
                        continue
                    else:
                        cds_pos1 = exon_end - cds_st
                        break
                for exon_size, exon_end in zip(exon_sizes,exon_ends):
                    if cds_end > exon_size:
                        cds_end = cds_end - exon_size
                        continue
                    else:
                        cds_pos2 = exon_end - cds_end - 3
                        break
                fields[7] = str(cds_pos1)
                fields[6] = str(cds_pos2)
                return '\t'.join(fields)



#!/usr/bin/env python
'''
Check if sequencing depth is saturated or not, based on the concept that when sequencing depth is 
approaching saturation, less NEW junctions will be discovered. 
See http://rseqc.sourceforge.net/ for details.
'''

#import built-in modules
import os,sys

if sys.version_info[0] != 3:
	print("\nYou are using python" + str(sys.version_info[0]) + '.' + str(sys.version_info[1]) + " This verion of RSeQC needs python3!\n", file=sys.stderr)
	sys.exit()	


import re
import string
from optparse import OptionParser
import warnings
import string
import collections
import math
from time import strftime
import subprocess

#import my own modules
from qcmodule import SAM
#changes to the paths

#changing history to this module


__author__ = "Liguo Wang"
__copyright__ = "Copyleft"
__credits__ = []
__license__ = "GPL"
__version__="3.0.0"
__maintainer__ = "Liguo Wang"
__email__ = "wang.liguo@mayo.edu"
__status__ = "Production"


def printlog (mesg):
	'''print progress into stderr and log file'''
	mesg="@ " + strftime("%Y-%m-%d %H:%M:%S") + ": " + mesg
	LOG=open('class.log','a')
	print(mesg, file=sys.stderr)
	print(mesg, file=LOG)


def main():
	usage="%prog [options]" + '\n' + __doc__ + "\n"
	parser = OptionParser(usage,version="%prog " + __version__)
	parser.add_option("-i","--input-file",action="store",type="string",dest="input_file",help="Alignment file in BAM or SAM format.[required]")
	parser.add_option("-o","--out-prefix",action="store",type="string",dest="output_prefix",help="Prefix of output files(s). [required]")
	parser.add_option("-r","--refgene",action="store",type="string",dest="refgene_bed",help="Reference gene model in bed fomat. This gene model is used to determine known splicing junctions. [required]")
	parser.add_option("-l","--percentile-floor",action="store",type="int",dest="percentile_low_bound",default=5, help="Sampling starts from this percentile. A integer between 0 and 100. default=%default")
	parser.add_option("-u","--percentile-ceiling",action="store",type="int",dest="percentile_up_bound",default=100, help="Sampling ends at this percentile. A integer between 0 and 100. default=%default")
	parser.add_option("-s","--percentile-step",action="store",type="int",dest="percentile_step",default=5, help="Sampling frequency. Smaller value means more sampling times. A integer between 0 and 100. default=%default")	
	parser.add_option("-m","--min-intron",action="store",type="int",dest="minimum_intron_size",default=50, help="Minimum intron size (bp). default=%default")
	parser.add_option("-v","--min-coverage",action="store",type="int",dest="minimum_splice_read",default=1, help="Minimum number of supportting reads to call a junction. default=%default")
	parser.add_option("-q","--mapq",action="store",type="int",dest="map_qual",default=30,help="Minimum mapping quality (phred scaled) for an alignment to be called \"uniquely mapped\". default=%default")

	(options,args)=parser.parse_args()

	if not (options.output_prefix and options.input_file and options.refgene_bed):
		parser.print_help()
		sys.exit(0)
	if options.percentile_low_bound <0 or options.percentile_low_bound >100:
		print("percentile_low_bound must be larger than 0 and samller than 100", file=sys.stderr)
		sys.exit(0)
	if options.percentile_up_bound <0 or options.percentile_up_bound >100:
		print("percentile_up_bound must be larger than 0 and samller than 100", file=sys.stderr)
		sys.exit(0)
	if options.percentile_up_bound < options.percentile_low_bound:
		print("percentile_up_bound must be larger than percentile_low_bound", file=sys.stderr)
		sys.exit(0)
	if options.percentile_step <0 or options.percentile_step > options.percentile_up_bound:
		print("percentile_step must be larger than 0 and samller than percentile_up_bound", file=sys.stderr)
		sys.exit(0)
	if os.path.exists(options.input_file):
		obj = SAM.ParseBAM(options.input_file)
		obj.saturation_junction(outfile=options.output_prefix, refgene=options.refgene_bed, sample_start=options.percentile_low_bound,sample_end=options.percentile_up_bound,sample_step=options.percentile_step,min_intron=options.minimum_intron_size,recur=options.minimum_splice_read, q_cut = options.map_qual)
		try:
			subprocess.call("Rscript " + options.output_prefix + '.junctionSaturation_plot.r', shell=True)
		except:
			print("Cannot generate pdf file from " + '.junctionSaturation_plot.r', file=sys.stderr)
			pass
	else:
		print('\n\n' + options.input_file + " does NOT exists" + '\n', file=sys.stderr)
		sys.exit(0)
		#parser.print_help()
		


if __name__ == '__main__':
	main()
